@@ -18,7 +18,7 @@ def check_dir( bucket , prefix ):
     zips = {}
     md5s = {}
     for ( key , modify , etag , size ) in bucket.listdir( prefix=prefix ):
-        if key.endswith( ".tgz" ) or key.endswith( ".zip" ):
+        if key.endswith( ".tgz" ) or key.endswith( ".zip" ) or key.endswith( ".tar.gz" ):
             zips[key] = etag.replace( '"' , '' )
         elif key.endswith( ".md5" ):
             md5s[key] = True
@@ -40,7 +40,7 @@ def run():
 
     bucket = simples3.S3Bucket( settings.bucket , settings.id , settings.key )
     
-    for x in [ "osx" , "linux" , "win32" , "sunos5" ]:
+    for x in [ "osx" , "linux" , "win32" , "sunos5" , "src" ]:
         check_dir( bucket , x )
     
 
