@@ -13,7 +13,10 @@ db.eval9.save( { a : 17 } );
 assert.eq( 1 , db.eval( "return db.eval9.find().toArray()" ).length , "A" );
 assert.eq( 17 , db.eval( "return db.eval9.find().toArray()" )[0].a , "B" );
 
-// just to make sure these things don't crash
-assert( db.eval( "return db.eval9.find()" ) );
-assert( db.eval( "return db.eval9" ) );
-assert( db.eval( "return db" ) );
+// just to make sure these things don't crash (but may throw an exception)
+try {
+    db.eval( "return db.eval9.find()" );
+    db.eval( "return db.eval9" );
+    db.eval( "return db" );
+    db.eval( "return print" );
+} catch (ex) { }
\ No newline at end of file
