@@ -1,5 +1,14 @@
 // Test persistence of list of dbs to add.
 
+var getDBNamesNoThrow = function(conn) {
+    try {
+        return conn.getDBNames();
+    } catch (e) {
+        printjson(e);
+        return [""];
+    }
+}
+
 doTest = function(signal, extraOpts) {
 
     rt = new ReplTest( "repl7tests" );
@@ -13,7 +22,7 @@ doTest = function(signal, extraOpts) {
     s = rt.start(false, extraOpts);
     
     assert.soon( function() {
-                return -1 != s.getDBNames().indexOf( "aa" );
+                     return -1 != getDBNamesNoThrow(s).indexOf( "aa" );
                 }, "aa timeout", 60000, 1000 );
     
     rt.stop( false, signal );
@@ -22,7 +31,7 @@ doTest = function(signal, extraOpts) {
     
     assert.soon( function() {
                 for( n = "a"; n != "aaaaa"; n += "a" ) {
-                    if ( -1 == s.getDBNames().indexOf( n ) )
+                    if ( -1 == getDBNamesNoThrow(s).indexOf( n ) )
                         return false;                    
                 }
                 return true;
