@@ -35,7 +35,7 @@ removeShard = function(st, replTest) {
            var res = directdb.currentOp( { desc: /^clean/ } );
            print( "eliot: " + replTest.getPrimary() + "\t" + tojson(res) );
            return res.inprog.length == 0;
-       }, "never clean", 60000, 1000 );
+       }, "never clean", 5 * 60 * 1000, 1000 );
     
     replTest.getPrimary().getDB( coll.getDB().getName() ).dropDatabase();
     print( "Shard removed successfully" );
@@ -59,12 +59,7 @@ addShard = function(st, replTest) {
         var x = st.chunkDiff( coll.getName() , coll.getDB().getName() );
         print( "chunk diff: " + x );
         return x < 2;
-    } , "no balance happened", 60000 );
-
-    // NOTE: To avoid getMore problems stranding cursors b/c of our current conn
-    // pool behavior, this needs to be run.
-    // TODO: Remove once conn pooling behavior is better
-    printjson( coll.getMongo().getDB("admin").runCommand({ connPoolSync : true }) );
+    } , "no balance happened", 5 * 60 * 1000 );
 
     try {
         assert.eq( 300, coll.find().itcount() );
@@ -119,7 +114,7 @@ assert.eq( 300, coll.find().itcount() );
 
 assert.soon( function() {
     var x = st.chunkDiff( 'remove2' , "test" ); print( "chunk diff: " + x ); return x < 2;
-} , "no balance happened" );
+} , "no balance happened", 5 * 60 * 1000 );
 
 assert.eq( 300, coll.find().itcount() );
 
