@@ -1,7 +1,7 @@
 
 
 s = new ShardingTest( "rs1" , 2 /* numShards */, 1 /* verboseLevel */, 1 /* numMongos */, { rs : true , numReplicas : 2 , chunksize : 1 , nopreallocj : true } )
-s.config.settings.update( { _id: "balancer" }, { $set : { stopped: true, _nosleep: true, replThrottle : true } } , true );
+s.config.settings.update( { _id: "balancer" }, { $set : { stopped: true, _nosleep: true, _secondaryThrottle : true } } , true );
 
 
 db = s.getDB( "test" );
@@ -26,7 +26,11 @@ coll.setSlaveOk();
 
 
 for ( i=0; i<20; i++ ) {
-    s.adminCommand( { moveChunk : "test.foo" , find : { _id : i * 100 } , to : other._id , _secondaryThrottle : true } );
+    // Needs to waitForDelete because we'll be performing a slaveOk query,
+    // and secondaries don't have a chunk manager so it doesn't know how to
+    // filter out docs it doesn't own.
+    s.adminCommand({ moveChunk: "test.foo", find: { _id: i * 100 }, to : other._id,
+        _secondaryThrottle: true, _waitForDelete: true });
     assert.eq( 2100, coll.find().itcount() );
 }
 
