@@ -1,4 +1,8 @@
 
+load( "jstests/libs/slow_weekly_util.js" )
+testServer = new SlowWeeklyMongod( "update_yield1" )
+db = testServer.getDB( "test" );
+
 t = db.update_yield1;
 t.drop()
 
@@ -73,9 +77,20 @@ printjson(x);
 assert.eq(1, x.inprog.length, "never doing update 2");
 assert.eq("update", x.inprog[0].op);
 
-t.findOne(); // should wait for update to finish
+while ( 1 ) {
+    t.findOne(); // should wait for update to finish
+    
+    var x = db.currentOp()
+    if ( x.inprog.length == 0 )
+        break;
 
-var x = db.currentOp()
-assert.eq( [] , x.inprog , "should have been atomic" );
+    assert( x.inprog.length == 1 && x.inprog[0].op == "update" , tojson( x ) );
+    
+    assert( x.inprog[0].numYields == 0 , tojson( x ) );
+
+    sleep( 100 );
+}
 
 join();
+
+testServer.stop();
