@@ -1,6 +1,7 @@
 /** @file dbcommands_generic.cpp commands suited for any mongo server (both mongod, mongos) */
 
 /**
+*    Copyright (C) 2012 10gen Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,12 +17,16 @@
 */
 
 #include "pch.h"
+
+#include "mongo/db/auth/action_set.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_manager.h"
+#include "mongo/db/auth/privilege.h"
 #include "pdfile.h"
 #include "jsobj.h"
 #include "../bson/util/builder.h"
 #include <time.h>
 #include "introspect.h"
-#include "btree.h"
 #include "../client/dbclient_rs.h"
 #include "../util/lruishmap.h"
 #include "../util/md5.hpp"
@@ -34,14 +39,12 @@
 #include "db.h"
 #include "instance.h"
 #include "lasterror.h"
-#include "security.h"
 #include "../scripting/engine.h"
 #include "stats/counters.h"
 #include "background.h"
 #include "../util/version.h"
 #include "../util/ramlog.h"
 #include "repl/multicmd.h"
-#include "mongo/s/shard.h"
 #include "server.h"
 
 namespace mongo {
@@ -106,177 +109,27 @@ namespace mongo {
         virtual bool adminOnly() const { return false; }
         virtual bool requiresAuth() { return false; }
         virtual LockType locktype() const { return NONE; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         virtual void help( stringstream &help ) const {
             help << "get version #, etc.\n";
             help << "{ buildinfo:1 }";
         }
-        bool run(const string& dbname, BSONObj& jsobj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl ) {
-            result << "version" << versionString << "gitVersion" << gitVersion() << "sysInfo" << sysInfo();
-            result << "versionArray" << versionArray;
-            result << "bits" << ( sizeof( int* ) == 4 ? 32 : 64 );
-            result.appendBool( "debug" , debug );
-            result.appendNumber("maxBsonObjectSize", BSONObjMaxUserSize);
-            return true;
+
+        bool run(const std::string& dbname,
+                 BSONObj& jsobj,
+                 int, // options
+                 std::string& errmsg,
+                 BSONObjBuilder& result,
+                 bool fromRepl) {
+        appendBuildInfo(result);
+        return true;
+
         }
+
     } cmdBuildInfo;
 
-    /** experimental. either remove or add support in repl sets also.  in a repl set, getting this setting from the
-        repl set config could make sense.
-        */
-    unsigned replApplyBatchSize = 1;
-
-    const char* fetchReplIndexPrefetchParam();
-
-    class CmdGet : public Command {
-    public:
-        CmdGet() : Command( "getParameter" ) { }
-        virtual bool slaveOk() const { return true; }
-        virtual bool adminOnly() const { return true; }
-        virtual LockType locktype() const { return NONE; }
-        virtual void help( stringstream &help ) const {
-            help << "get administrative option(s)\nexample:\n";
-            help << "{ getParameter:1, notablescan:1 }\n";
-            help << "supported so far:\n";
-            help << "  quiet\n";
-            help << "  notablescan\n";
-            help << "  logLevel\n";
-            help << "  syncdelay\n";
-            help << "{ getParameter:'*' } to get everything\n";
-        }
-        bool run(const string& dbname, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl ) {
-            bool all = *cmdObj.firstElement().valuestrsafe() == '*';
-
-            int before = result.len();
-
-            if( all || cmdObj.hasElement("quiet") ) {
-                result.append("quiet", cmdLine.quiet );
-            }
-            if( all || cmdObj.hasElement("notablescan") ) {
-                result.append("notablescan", cmdLine.noTableScan);
-            }
-            if( all || cmdObj.hasElement("logLevel") ) {
-                result.append("logLevel", logLevel);
-            }
-            if( all || cmdObj.hasElement("syncdelay") ) {
-                result.append("syncdelay", cmdLine.syncdelay);
-            }
-            if( all || cmdObj.hasElement("replApplyBatchSize") ) {
-                result.append("replApplyBatchSize", replApplyBatchSize);
-            }
-            if (all || cmdObj.hasElement("replIndexPrefetch")) {
-                result.append("replIndexPrefetch", fetchReplIndexPrefetchParam());
-            }
-            if (all || cmdObj.hasElement("releaseConnectionsAfterResponse")) {
-                result.append("releaseConnectionsAfterResponse", 
-                              ShardConnection::releaseConnectionsAfterResponse);
-            }
-            if ( before == result.len() ) {
-                errmsg = "no option found to get";
-                return false;
-            }
-            return true;
-        }
-    } cmdGet;
-
-    // tempish
-    bool setParmsMongodSpecific(const string& dbname, BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool fromRepl );
-
-    class CmdSet : public Command {
-    public:
-        CmdSet() : Command( "setParameter" ) { }
-        virtual bool slaveOk() const { return true; }
-        virtual bool adminOnly() const { return true; }
-        virtual LockType locktype() const { return NONE; }
-        virtual void help( stringstream &help ) const {
-            help << "set administrative option(s)\n";
-            help << "{ setParameter:1, <param>:<value> }\n";
-            help << "supported so far:\n";
-            help << "  journalCommitInterval\n";
-            help << "  logLevel\n";
-            help << "  notablescan\n";
-            help << "  quiet\n";
-            help << "  syncdelay\n";
-        }
-        bool run(const string& dbname, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl ) {
-            int s = 0;
-            bool found = setParmsMongodSpecific(dbname, cmdObj, errmsg, result, fromRepl);
-            if( cmdObj.hasElement("journalCommitInterval") ) { 
-                if( !cmdLine.dur ) { 
-                    errmsg = "journaling is off";
-                    return false;
-                }
-                int x = (int) cmdObj["journalCommitInterval"].Number();
-                verify( x > 1 && x < 500 );
-                cmdLine.journalCommitInterval = x;
-                log() << "setParameter journalCommitInterval=" << x << endl;
-                s++;
-            }
-            if( cmdObj.hasElement("notablescan") ) {
-                verify( !cmdLine.isMongos() );
-                if( s == 0 )
-                    result.append("was", cmdLine.noTableScan);
-                cmdLine.noTableScan = cmdObj["notablescan"].Bool();
-                s++;
-            }
-            if( cmdObj.hasElement("quiet") ) {
-                if( s == 0 )
-                    result.append("was", cmdLine.quiet );
-                cmdLine.quiet = cmdObj["quiet"].Bool();
-                s++;
-            }
-            if( cmdObj.hasElement("syncdelay") ) {
-                verify( !cmdLine.isMongos() );
-                if( s == 0 )
-                    result.append("was", cmdLine.syncdelay );
-                cmdLine.syncdelay = cmdObj["syncdelay"].Number();
-                s++;
-            }
-            if( cmdObj.hasElement( "logLevel" ) ) {
-                if( s == 0 )
-                    result.append("was", logLevel );
-                logLevel = cmdObj["logLevel"].numberInt();
-                s++;
-            }
-            if( cmdObj.hasElement( "replApplyBatchSize" ) ) {
-                if( s == 0 )
-                    result.append("was", replApplyBatchSize );
-                BSONElement e = cmdObj["replApplyBatchSize"];
-                ParameterValidator * v = ParameterValidator::get( e.fieldName() );
-                verify( v );
-                if ( ! v->isValid( e , errmsg ) )
-                    return false;
-                replApplyBatchSize = e.numberInt();
-                s++;
-            }
-            if( cmdObj.hasElement( "traceExceptions" ) ) {
-                if( s == 0 ) result.append( "was", DBException::traceExceptions );
-                DBException::traceExceptions = cmdObj["traceExceptions"].Bool();
-                s++;
-            }
-            if( cmdObj.hasElement( "replMonitorMaxFailedChecks" ) ) {
-                if( s == 0 ) result.append( "was", ReplicaSetMonitor::getMaxFailedChecks() );
-                ReplicaSetMonitor::setMaxFailedChecks(
-                        cmdObj["replMonitorMaxFailedChecks"].numberInt() );
-                s++;
-            }
-            if( cmdObj.hasElement( "releaseConnectionsAfterResponse" ) ) {
-                if ( s == 0 ) {
-                    result.append( "was", 
-                                   ShardConnection::releaseConnectionsAfterResponse );
-                }
-                ShardConnection::releaseConnectionsAfterResponse = 
-                    cmdObj["releaseConnectionsAfterResponse"].trueValue();
-                s++;
-            }
-
-            if( s == 0 && !found ) {
-                errmsg = "no option found to set, use help:true to see options ";
-                return false;
-            }
-
-            return true;
-        }
-    } cmdSet;
 
     class PingCommand : public Command {
     public:
@@ -285,6 +138,9 @@ namespace mongo {
         virtual void help( stringstream &help ) const { help << "a way to check that the server is alive. responds immediately even if server is in a db lock."; }
         virtual LockType locktype() const { return NONE; }
         virtual bool requiresAuth() { return false; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         virtual bool run(const string& badns, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
             // IMPORTANT: Don't put anything in here that might lock db - including authentication
             return true;
@@ -298,6 +154,9 @@ namespace mongo {
         virtual bool slaveOk() const { return true; }
         virtual bool readOnly() { return true; }
         virtual LockType locktype() const { return NONE; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         virtual bool run(const string& ns, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
             if ( globalScriptEngine ) {
                 BSONObjBuilder bb( result.subobjStart( "js" ) );
@@ -326,7 +185,13 @@ namespace mongo {
         virtual void help( stringstream& help ) const {
             help << "returns information about the daemon's host";
         }
-
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {
+            ActionSet actions;
+            actions.addAction(ActionType::hostInfo);
+            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+        }
         bool run(const string& dbname, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
             ProcessInfo p;
             BSONObjBuilder bSys, bOs;
@@ -357,9 +222,15 @@ namespace mongo {
         virtual LockType locktype() const { return NONE; }
         virtual bool slaveOk() const { return true; }
         virtual bool adminOnly() const { return true; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {
+            ActionSet actions;
+            actions.addAction(ActionType::logRotate);
+            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+        }
         virtual bool run(const string& ns, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
-            fassert(16175, rotateLogs());
-            return 1;
+            return rotateLogs();
         }
 
     } logRotateCmd;
@@ -371,6 +242,9 @@ namespace mongo {
         virtual LockType locktype() const { return NONE; }
         virtual bool slaveOk() const { return true; }
         virtual bool adminOnly() const { return false; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         virtual bool run(const string& ns, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
             BSONObjBuilder b( result.subobjStart( "commands" ) );
             for ( map<string,Command*>::iterator i=_commands->begin(); i!=_commands->end(); ++i ) {
@@ -427,6 +301,9 @@ namespace mongo {
             return true;
         }
         virtual LockType locktype() const { return NONE; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         CmdForceError() : Command("forceerror") {}
         bool run(const string& dbnamne, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
             uassert( 10038 , "forced error", false);
@@ -439,6 +316,10 @@ namespace mongo {
         AvailableQueryOptions() : Command( "availableQueryOptions" , false , "availablequeryoptions" ) {}
         virtual bool slaveOk() const { return true; }
         virtual LockType locktype() const { return NONE; }
+        virtual bool requiresAuth() { return false; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {} // No auth required
         virtual bool run(const string& dbname , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
             result << "options" << QueryOption_AllSupported;
             return true;
@@ -452,7 +333,13 @@ namespace mongo {
         virtual bool slaveOk() const { return true; }
         virtual LockType locktype() const { return NONE; }
         virtual bool adminOnly() const { return true; }
-
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {
+            ActionSet actions;
+            actions.addAction(ActionType::getLog);
+            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+        }
         virtual void help( stringstream& help ) const {
             help << "{ getLog : '*' }  OR { getLog : 'global' }";
         }
@@ -476,10 +363,12 @@ namespace mongo {
                     errmsg = str::stream() << "no RamLog named: " << p;
                     return false;
                 }
-                
+
+                result.appendNumber( "totalLinesWritten", rl->getTotalLinesWritten() );
+
                 vector<const char*> lines;
                 rl->get( lines );
-                
+
                 BSONArrayBuilder arr( result.subarrayStart( "log" ) );
                 for ( unsigned i=0; i<lines.size(); i++ )
                     arr.append( lines[i] );
@@ -490,4 +379,26 @@ namespace mongo {
 
     } getLogCmd;
 
+    class CmdGetCmdLineOpts : Command {
+    public:
+        CmdGetCmdLineOpts(): Command("getCmdLineOpts") {}
+        void help(stringstream& h) const { h << "get argv"; }
+        virtual LockType locktype() const { return NONE; }
+        virtual bool adminOnly() const { return true; }
+        virtual bool slaveOk() const { return true; }
+        virtual void addRequiredPrivileges(const std::string& dbname,
+                                           const BSONObj& cmdObj,
+                                           std::vector<Privilege>* out) {
+            ActionSet actions;
+            actions.addAction(ActionType::getCmdLineOpts);
+            out->push_back(Privilege(AuthorizationManager::SERVER_RESOURCE_NAME, actions));
+        }
+        virtual bool run(const string&, BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool fromRepl) {
+            result.append("argv", CmdLine::getArgvArray());
+            result.append("parsed", CmdLine::getParsedOpts());
+            return true;
+        }
+
+    } cmdGetCmdLineOpts;
+
 }
