@@ -16,17 +16,22 @@
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "mongo/pch.h"
+
+#include "mongo/db/index.h"
+
 #include <boost/checked_delete.hpp>
 
-#include "pch.h"
-#include "namespace-inl.h"
-#include "index.h"
-#include "btree.h"
-#include "background.h"
-#include "repl/rs.h"
-#include "ops/delete.h"
+#include "mongo/db/auth/action_type.h"
+#include "mongo/db/auth/authorization_manager.h"
+#include "mongo/db/background.h"
+#include "mongo/db/btree.h"
+#include "mongo/db/index_update.h"
+#include "mongo/db/namespace-inl.h"
+#include "mongo/db/ops/delete.h"
+#include "mongo/db/repl/rs.h"
 #include "mongo/util/scopeguard.h"
-
+#include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
 
@@ -245,7 +250,7 @@ namespace mongo {
 
     void getIndexChanges(vector<IndexChanges>& v, const char *ns, NamespaceDetails& d,
                          BSONObj newObj, BSONObj oldObj, bool &changedId) {
-        int z = d.nIndexesBeingBuilt();
+        int z = d.getTotalIndexCount();
         v.resize(z);
         for( int i = 0; i < z; i++ ) {
             IndexDetails& idx = d.idx(i);
@@ -264,7 +269,7 @@ namespace mongo {
     }
 
     void dupCheck(vector<IndexChanges>& v, NamespaceDetails& d, DiskLoc curObjLoc) {
-        int z = d.nIndexesBeingBuilt();
+        int z = d.getTotalIndexCount();
         for( int i = 0; i < z; i++ ) {
             IndexDetails& idx = d.idx(i);
             v[i].dupCheck(idx, curObjLoc);
@@ -282,32 +287,60 @@ namespace mongo {
         return true;
     }
 
-    /* Prepare to build an index.  Does not actually build it (except for a special _id case).
-       - We validate that the params are good
-       - That the index does not already exist
-       - Creates the source collection if it DNE
+    static bool needToUpgradeMinorVersion(const string& newPluginName) {
+        if (IndexPlugin::existedBefore24(newPluginName))
+            return false;
 
-       example of 'io':
-         { ns : 'test.foo', name : 'z', key : { z : 1 } }
+        DataFileHeader* dfh = cc().database()->getFile(0)->getHeader();
+        if (dfh->versionMinor == PDFILE_VERSION_MINOR_24_AND_NEWER)
+            return false; // these checks have already been done
 
-       throws DBException
+        fassert(16737, dfh->versionMinor == PDFILE_VERSION_MINOR_22_AND_OLDER);
 
-       @param sourceNS - source NS we are indexing
-       @param sourceCollection - its details ptr
-       @return true if ok to continue.  when false we stop/fail silently (index already exists)
-    */
-    bool prepareToBuildIndex(const BSONObj& io, bool god, string& sourceNS, NamespaceDetails *&sourceCollection, BSONObj& fixedIndexObject ) {
+        return true;
+    }
+
+    static void upgradeMinorVersionOrAssert(const string& newPluginName) {
+        const string systemIndexes = cc().database()->name + ".system.indexes";
+        shared_ptr<Cursor> cursor(theDataFileMgr.findAll(systemIndexes));
+        for ( ; cursor && cursor->ok(); cursor->advance()) {
+            const BSONObj index = cursor->current();
+            const BSONObj key = index.getObjectField("key");
+            const string plugin = IndexPlugin::findPluginName(key);
+            if (IndexPlugin::existedBefore24(plugin))
+                continue;
+
+            const string errmsg = str::stream()
+                << "Found pre-existing index " << index << " with invalid type '" << plugin << "'. "
+                << "Disallowing creation of new index type '" << newPluginName << "'. See "
+                << "http://dochub.mongodb.org/core/index-type-changes"
+                ;
+
+            error() << errmsg << endl;
+            uasserted(16738, errmsg);
+        }
+
+        DataFileHeader* dfh = cc().database()->getFile(0)->getHeader();
+        getDur().writingInt(dfh->versionMinor) = PDFILE_VERSION_MINOR_24_AND_NEWER;
+    }
+
+    bool prepareToBuildIndex(const BSONObj& io,
+                             bool mayInterrupt,
+                             bool god,
+                             string& sourceNS,
+                             NamespaceDetails*& sourceCollection,
+                             BSONObj& fixedIndexObject) {
         sourceCollection = 0;
 
-        // logical name of the index.  todo: get rid of the name, we don't need it!
-        const char *name = io.getStringField("name");
-        uassert(12523, "no index name specified", *name);
-
         // the collection for which we are building an index
         sourceNS = io.getStringField("ns");
         uassert(10096, "invalid ns to index", sourceNS.find( '.' ) != string::npos);
         massert(10097, str::stream() << "bad table to index name on add index attempt current db: " << cc().database()->name << "  source: " << sourceNS ,
-                cc().database()->name == nsToDatabase(sourceNS.c_str()));
+                cc().database()->name == nsToDatabase(sourceNS));
+
+        // logical name of the index.  todo: get rid of the name, we don't need it!
+        const char *name = io.getStringField("name");
+        uassert(12523, "no index name specified", *name);
 
         BSONObj key = io.getObjectField("key");
         uassert(12524, "index key pattern too large", key.objsize() <= 2048);
@@ -323,7 +356,7 @@ namespace mongo {
             uasserted(12504, s);
         }
 
-        sourceCollection = nsdetails(sourceNS.c_str());
+        sourceCollection = nsdetails(sourceNS);
         if( sourceCollection == 0 ) {
             // try to create it
             string err;
@@ -331,16 +364,19 @@ namespace mongo {
                 problem() << "ERROR: failed to create collection while adding its index. " << sourceNS << endl;
                 return false;
             }
-            sourceCollection = nsdetails(sourceNS.c_str());
+            sourceCollection = nsdetails(sourceNS);
             tlog() << "info: creating collection " << sourceNS << " on add index" << endl;
             verify( sourceCollection );
         }
 
-        if ( sourceCollection->findIndexByName(name) >= 0 ) {
+        // Check both existing and in-progress indexes (2nd param = true)
+        if ( sourceCollection->findIndexByName(name, true) >= 0 ) {
             // index already exists.
             return false;
         }
-        if( sourceCollection->findIndexByKeyPattern(key) >= 0 ) {
+
+        // Check both existing and in-progress indexes (2nd param = true)
+        if( sourceCollection->findIndexByKeyPattern(key, true) >= 0 ) {
             LOG(2) << "index already exists with diff name " << name << ' ' << key.toString() << endl;
             return false;
         }
@@ -349,22 +385,16 @@ namespace mongo {
             stringstream ss;
             ss << "add index fails, too many indexes for " << sourceNS << " key:" << key.toString();
             string s = ss.str();
-            log() << s << '\n';
+            log() << s << endl;
             uasserted(12505,s);
         }
 
-        /* we can't build a new index for the ns if a build is already in progress in the background -
-           EVEN IF this is a foreground build.
-           */
-        uassert(12588, "cannot add index with a background operation in progress",
-                !BackgroundOperation::inProgForNs(sourceNS.c_str()));
-
         /* this is because we want key patterns like { _id : 1 } and { _id : <someobjid> } to
            all be treated as the same pattern.
         */
         if ( IndexDetails::isIdIndexPattern(key) ) {
             if( !god ) {
-                ensureHaveIdIndex( sourceNS.c_str() );
+                ensureHaveIdIndex( sourceNS.c_str(), mayInterrupt );
                 return false;
             }
         }
@@ -377,8 +407,16 @@ namespace mongo {
         }
 
         string pluginName = IndexPlugin::findPluginName( key );
-        IndexPlugin * plugin = pluginName.size() ? IndexPlugin::get( pluginName ) : 0;
+        IndexPlugin * plugin = NULL;
+        if (pluginName.size()) {
+            plugin = IndexPlugin::get(pluginName);
+            uassert(16734, str::stream() << "Unknown index plugin '" << pluginName << "' "
+                                         << "in index "<< key
+                   , plugin);
 
+            if (needToUpgradeMinorVersion(pluginName))
+                upgradeMinorVersionOrAssert(pluginName);
+        }
 
         { 
             BSONObj o = io;
@@ -419,19 +457,25 @@ namespace mongo {
         return true;
     }
 
-    void IndexSpec::reset( const IndexDetails * details ) {
+    void IndexSpec::reset(const IndexDetails * details) {
+        const DataFileHeader* dfh = cc().database()->getFile(0)->getHeader();
+        IndexSpec::PluginRules rules = dfh->versionMinor == PDFILE_VERSION_MINOR_24_AND_NEWER
+                                            ? IndexSpec::RulesFor24
+                                            : IndexSpec::RulesFor22
+                                            ;
+
         _details = details;
-        reset( details->info );
+        reset(details->info, rules);
     }
 
-    void IndexSpec::reset( const BSONObj& _info ) {
+    void IndexSpec::reset(const BSONObj& _info, PluginRules rules) {
         info = _info;
         keyPattern = info["key"].embeddedObjectUserCheck();
         if ( keyPattern.objsize() == 0 ) {
             out() << info.toString() << endl;
             verify(false);
         }
-        _init();
+        _init(rules);
     }
 
     void IndexChanges::dupCheck(IndexDetails& idx, DiskLoc curObjLoc) {
