@@ -54,7 +54,6 @@ namespace mongo {
     static UpdateResult _updateById(bool isOperatorUpdate,
                                     int idIdxNo,
                                     ModSet* mods,
-                                    int profile,
                                     NamespaceDetails* d,
                                     NamespaceDetailsTransient *nsdt,
                                     bool su,
@@ -85,10 +84,11 @@ namespace mongo {
            regular ones at the moment. */
         if ( isOperatorUpdate ) {
             const BSONObj& onDisk = loc.obj();
-            auto_ptr<ModSetState> mss = mods->prepare( onDisk );
+            auto_ptr<ModSetState> mss = mods->prepare( onDisk, false /* not an insertion */ );
 
             if( mss->canApplyInPlace() ) {
                 mss->applyModsInPlace(true);
+                debug.fastmod = true;
                 DEBUGUPDATE( "\t\t\t updateById doing in place update" );
             }
             else {
@@ -146,7 +146,6 @@ namespace mongo {
                      << " upsert: " << upsert << " multi: " << multi );
 
         Client& client = cc();
-        int profile = client.database()->profile;
 
         debug.updateobj = updateobj;
 
@@ -161,15 +160,8 @@ namespace mongo {
         bool isOperatorUpdate = updateobj.firstElementFieldName()[0] == '$';
         int modsIsIndexed = false; // really the # of indexes
         if ( isOperatorUpdate ) {
-            if( d && d->indexBuildInProgress ) {
-                set<string> bgKeys;
-                d->inProgIdx().keyPattern().getFieldNames(bgKeys);
-                mods.reset( new ModSet(updateobj, nsdt->indexKeys(), &bgKeys, forReplication) );
-            }
-            else {
-                mods.reset( new ModSet(updateobj, nsdt->indexKeys(), NULL, forReplication) );
-            }
-            modsIsIndexed = mods->isIndexed();
+            mods.reset( new ModSet(updateobj, nsdt->indexKeys(), forReplication) );
+            modsIsIndexed = mods->maxNumIndexUpdated();
         }
 
         if( planPolicy.permitOptimalIdPlan() && !multi && isSimpleIdQuery(patternOrig) && d &&
@@ -181,7 +173,6 @@ namespace mongo {
                 UpdateResult result = _updateById( isOperatorUpdate,
                                                    idxNo,
                                                    mods.get(),
-                                                   profile,
                                                    d,
                                                    nsdt,
                                                    su,
@@ -194,12 +185,15 @@ namespace mongo {
                 if ( result.existing || ! upsert ) {
                     return result;
                 }
-                else if ( upsert && ! isOperatorUpdate && ! logop) {
+                else if ( upsert && ! isOperatorUpdate ) {
                     // this handles repl inserts
                     checkNoMods( updateobj );
                     debug.upsert = true;
                     BSONObj no = updateobj;
-                    theDataFileMgr.insertWithObjMod(ns, no, su);
+                    theDataFileMgr.insertWithObjMod(ns, no, false, su);
+                    if ( logop )
+                        logOp( "i", ns, no, 0, 0, fromMigrate );
+
                     return UpdateResult( 0 , 0 , 1 , no );
                 }
             }
@@ -249,13 +243,9 @@ namespace mongo {
                         if ( ! d )
                             break;
                         nsdt = &NamespaceDetailsTransient::get(ns);
-                        if ( mods.get() && ! mods->isIndexed() ) {
-                            // we need to re-check indexes
-                            set<string> bgKeys;
-                            if ( d->indexBuildInProgress )
-                                d->inProgIdx().keyPattern().getFieldNames(bgKeys);
-                            mods->updateIsIndexed( nsdt->indexKeys() , &bgKeys );
-                            modsIsIndexed = mods->isIndexed();
+                        if ( mods.get() ) {
+                            mods->setIndexedStatus( nsdt->indexKeys() );
+                            modsIsIndexed = mods->maxNumIndexUpdated();
                         }
 
                     }
@@ -265,10 +255,6 @@ namespace mongo {
                 debug.nscanned++;
 
                 if ( mods.get() && mods->hasDynamicArray() ) {
-                    // The Cursor must have a Matcher to record an elemMatchKey.  But currently
-                    // a modifier on a dynamic array field may be applied even if there is no
-                    // elemMatchKey, so a matcher cannot be required.
-                    //verify( c->matcher() );
                     details.requestElemMatchKey();
                 }
 
@@ -339,7 +325,8 @@ namespace mongo {
                         mymodset.reset( useMods );
                     }
 
-                    auto_ptr<ModSetState> mss = useMods->prepare( onDisk );
+                    auto_ptr<ModSetState> mss = useMods->prepare( onDisk,
+                                                                  false /* not an insertion */ );
 
                     bool willAdvanceCursor = multi && c->ok() && ( modsIsIndexed || ! mss->canApplyInPlace() );
 
@@ -350,11 +337,17 @@ namespace mongo {
                         c->prepareToTouchEarlierIterate();
                     }
 
-                    if ( modsIsIndexed <= 0 && mss->canApplyInPlace() ) {
+                    // If we've made it this far, "ns" must contain a valid collection name, and so
+                    // is of the form "db.collection".  Therefore, the following expression must
+                    // always be valid.  "system.users" updates must never be done in place, in
+                    // order to ensure that they are validated inside DataFileMgr::updateRecord(.).
+                    bool isSystemUsersMod = (NamespaceString(ns).coll == "system.users");
+
+                    if ( !mss->isUpdateIndexed() && mss->canApplyInPlace() && !isSystemUsersMod ) {
                         mss->applyModsInPlace( true );// const_cast<BSONObj&>(onDisk) );
 
                         DEBUGUPDATE( "\t\t\t doing in place update" );
-                        if ( profile && !multi )
+                        if ( !multi )
                             debug.fastmod = true;
 
                         if ( modsIsIndexed ) {
@@ -364,9 +357,6 @@ namespace mongo {
                         d->paddingFits();
                     }
                     else {
-                        if ( rs )
-                            rs->goingToDelete( onDisk );
-
                         BSONObj newObj = mss->createNewFromMods();
                         checkTooLarge(newObj);
                         DiskLoc newLoc = theDataFileMgr.updateRecord(ns,
@@ -433,7 +423,7 @@ namespace mongo {
                 BSONObj newObj = mods->createNewFromQuery( patternOrig );
                 checkNoMods( newObj );
                 debug.fastmodinsert = true;
-                theDataFileMgr.insertWithObjMod(ns, newObj, su);
+                theDataFileMgr.insertWithObjMod(ns, newObj, false, su);
                 if ( logop )
                     logOp( "i", ns, newObj, 0, 0, fromMigrate );
 
@@ -443,7 +433,7 @@ namespace mongo {
             checkNoMods( updateobj );
             debug.upsert = true;
             BSONObj no = updateobj;
-            theDataFileMgr.insertWithObjMod(ns, no, su);
+            theDataFileMgr.insertWithObjMod(ns, no, false, su);
             if ( logop )
                 logOp( "i", ns, no, 0, 0, fromMigrate );
             return UpdateResult( 0 , 0 , 1 , no );
@@ -513,7 +503,7 @@ namespace mongo {
 
     BSONObj applyUpdateOperators( const BSONObj& from, const BSONObj& operators ) {
         ModSet mods( operators );
-        return mods.prepare( from )->createNewFromMods();
+        return mods.prepare( from, false /* not an insertion */ )->createNewFromMods();
     }
     
 }  // namespace mongo
