@@ -18,9 +18,12 @@
 
 #include "pch.h"
 
-#include "mongo/db/oplog.h"
+#include <algorithm> // for max
+
+#include "mongo/db/field_ref.h"
 #include "mongo/db/jsobjmanipulator.h"
 #include "mongo/db/pdfile.h"
+#include "mongo/db/oplog.h"
 #include "mongo/util/mongoutils/str.h"
 
 #include "update_internal.h"
@@ -31,7 +34,8 @@
 namespace mongo {
 
     const char* Mod::modNames[] = { "$inc", "$set", "$push", "$pushAll", "$pull", "$pullAll" , "$pop", "$unset" ,
-                                    "$bitand" , "$bitor" , "$bit" , "$addToSet", "$rename", "$rename"
+                                    "$bitand" , "$bitor" , "$bit" , "$addToSet", "$rename", "$rename" ,
+                                    "$setOnInsert"
                                   };
     unsigned Mod::modNamesNum = sizeof(Mod::modNames)/sizeof(char*);
 
@@ -105,6 +109,14 @@ namespace mongo {
             break;
         }
 
+        case SET_ON_INSERT:
+            // There is a corner case that would land us here (making a change to an existing
+            // field with $setOnInsert). If we're in an upsert, and the query portion of the
+            // update creates a field, we can modify it with $setOnInsert. This degenerates
+            // into a $set, so we fall through to the next case.
+            ms.fixedOpName = "$set";
+            // Fall through.
+
         case SET: {
             _checkForAppending( elt );
             builder.appendAs( elt , shortFieldName );
@@ -118,24 +130,143 @@ namespace mongo {
 
         case PUSH: {
             uassert( 10131 ,  "$push can only be applied to an array" , in.type() == Array );
+
+            //
+            // We can be in a single element push case, a "push all" case, or a "push all" case
+            // with a slice requirement (ie, a "push to size"). In each of these, we decide
+            // differently how much of the existing- and of the parameter-array to copy to the
+            // final object.
+            //
+
+            // Start the resulting array's builder.
             BSONArrayBuilder bb( builder.subarrayStart( shortFieldName ) );
-            BSONObjIterator i( in.embeddedObject() );
-            while ( i.more() ) {
-                bb.append( i.next() );
+
+            // If in the single element push case, we'll copy all elements of the existing
+            // array and add the new one.
+            if ( ! isEach() ) {
+                BSONObjIterator i( in.embeddedObject() );
+                while ( i.more() ) {
+                    bb.append( i.next() );
+                }
+                bb.append( elt );
+
+                // We don't want to log a positional $set for which the '_checkForAppending' test
+                // won't pass. If we're in that case, fall back to non-optimized logging.
+                if ( (elt.type() == Object && elt.embeddedObject().okForStorage()) ||
+                     (elt.type() != Object) ) {
+                    ms.fixedOpName = "$set";
+                    ms.forcePositional = true;
+                    ms.position = bb.arrSize() - 1;
+                    bb.done();
+                }
+                else {
+                    ms.fixedOpName = "$set";
+                    ms.forceEmptyArray = true;
+                    ms.fixedArray = BSONArray( bb.done().getOwned() );
+                }
             }
 
-            bb.append( elt );
+            // If we're in the "push all" case, we'll copy all element of both the existing and
+            // parameter arrays.
+            else if ( isEach() && ! isSliceOnly() && ! isSliceAndSort() ) {
+                BSONObjIterator i( in.embeddedObject() );
+                while ( i.more() ) {
+                    bb.append( i.next() );
+                }
+                BSONObjIterator j( getEach() );
+                while ( j.more() ) {
+                    bb.append( j.next() );
+                }
 
-            // We don't want to log a positional $set for which the '_checkForAppending' test
-            // won't pass. If we're in that case, fall back to non-optimized logging.
-            if ( (elt.type() == Object && elt.embeddedObject().okForStorage()) ||
-                 (elt.type() != Object) ) {
                 ms.fixedOpName = "$set";
-                ms.forcePositional = true;
-                ms.position = bb.arrSize() - 1;
-                bb.done();
+                ms.forceEmptyArray = true;
+                ms.fixedArray = BSONArray( bb.done().getOwned() );
             }
+
+            // If we're in the "push with a $each" case with slice, we have to decide how much
+            // of each of the existing and parameter arrays to copy to the final object.
+            else if ( isSliceOnly() ) {
+                long long slice = getSlice();
+                BSONObj eachArray = getEach();
+                long long arraySize = in.embeddedObject().nFields();
+                long long eachArraySize = eachArray.nFields();
+
+                // Zero slice is equivalent to resetting the array in the final object, so
+                // we won't copy anything.
+                if (slice == 0) {
+                    // no-op
+                }
+
+                // If the parameter array alone is larger than the slice, then only copy
+                // object from that array.
+                else if (slice <= eachArraySize) {
+                    long long skip = eachArraySize - slice;
+                    BSONObjIterator j( getEach() );
+                    while ( j.more() ) {
+                        if ( skip-- > 0 ) {
+                            j.next();
+                            continue;
+                        }
+                        bb.append( j.next() );
+                    }
+                }
+
+                // If the parameter array is not sufficient to fill the slice, then some (or all)
+                // the elements from the existing array will be copied too.
+                else {
+                    long long skip = std::max(0LL, arraySize - (slice - eachArraySize) );
+                    BSONObjIterator i( in.embeddedObject() );
+                    while ( i.more() ) {
+                        if (skip-- > 0) {
+                            i.next();
+                            continue;
+                        }
+                        bb.append( i.next() );
+                    }
+                    BSONObjIterator j( getEach() );
+                    while ( j.more() ) {
+                        bb.append( j.next() );
+                    }
+                }
+
+                ms.fixedOpName = "$set";
+                ms.forceEmptyArray = true;
+                ms.fixedArray = BSONArray( bb.done().getOwned() );
+            }
+
+            // If we're in the "push all" case ($push with a $each) with sort, we have to
+            // concatenate the existing array with the $each array, sort the result, and then
+            // decide how much of each of the resulting work area to copy to the final object.
             else {
+                long long slice = getSlice();
+
+                // Zero slice is equivalent to resetting the array in the final object, so
+                // we only go into sorting if there is anything to sort.
+                if ( slice > 0 ) {
+                    vector<BSONObj> workArea;
+                    BSONObjIterator i( in.embeddedObject() );
+                    while ( i.more() ) {
+                        workArea.push_back( i.next().Obj() );
+                    }
+                    BSONObjIterator j( getEach() );
+                    while ( j.more() ) {
+                        workArea.push_back( j.next().Obj() );
+                    }
+                    ProjectKeyCmp cmp( getSort() );
+                    sort( workArea.begin(), workArea.end(), cmp );
+
+                    long long skip = std::max( 0LL,
+                                               (long long)workArea.size() - slice );
+                    for ( vector<BSONObj>::iterator it = workArea.begin();
+                         it != workArea.end();
+                         ++it ) {
+                        if ( skip-- > 0 ) {
+                            continue;
+                        }
+                        bb.append( *it );
+                    }
+                }
+
                 ms.fixedOpName = "$set";
                 ms.forceEmptyArray = true;
                 ms.fixedArray = BSONArray( bb.done().getOwned() );
@@ -399,9 +530,11 @@ namespace mongo {
         return !obj.getField( path ).eoo();
     }
 
-    auto_ptr<ModSetState> ModSet::prepare(const BSONObj& obj) const {
+    auto_ptr<ModSetState> ModSet::prepare(const BSONObj& obj, bool insertion) const {
         DEBUGUPDATE( "\t start prepare" );
-        auto_ptr<ModSetState> mss( new ModSetState( obj ) );
+        auto_ptr<ModSetState> mss( new ModSetState( obj,
+                                                    _numIndexAlwaysUpdated,
+                                                    _numIndexMaybeUpdated ) );
 
 
         // Perform this check first, so that we don't leave a partially modified object on uassert.
@@ -411,6 +544,24 @@ namespace mongo {
             ModState& ms = *mss->_mods[i->first];
 
             const Mod& m = i->second;
+
+            // Check for any positional operators that have not been replaced with a numeric field
+            // name (from a query match element).
+            // Only perform this positional operator validation in 'strictApply' mode.  When
+            // replicating from a legacy primary that does not implement this validation, the
+            // secondary bypasses validation and remains consistent with the primary.
+            if ( m.strictApply ) {
+                FieldRef fieldRef;
+                fieldRef.parse( m.fieldName );
+                StringData positionalOpField( "$" );
+                for( size_t i = 0; i < fieldRef.numParts(); ++i ) {
+                     uassert( 16650,
+                              "Cannot apply the positional operator without a corresponding query "
+                              "field containing an array.",
+                              fieldRef.getPart( i ).compare( positionalOpField ) != 0 );
+                }
+            }
+
             BSONElement e = obj.getFieldDotted(m.fieldName);
 
             ms.m = &m;
@@ -439,7 +590,7 @@ namespace mongo {
                 continue;
             }
 
-            if ( e.eoo() ) {
+            if ( m.op != Mod::SET_ON_INSERT && e.eoo() ) {
                 mss->amIInPlacePossible( m.op == Mod::UNSET );
                 continue;
             }
@@ -466,11 +617,35 @@ namespace mongo {
                                          m.elt.valuesize() == e.valuesize() );
                 break;
 
+            case Mod::SET_ON_INSERT:
+                // If the document exist (i.e this is an update, not an insert) $setOnInsert
+                // becomes a no-op.
+                if ( !insertion ) {
+                    ms.dontApply = true;
+                    mss->amIInPlacePossible( true );
+                }
+                else {
+                    mss->amIInPlacePossible( false );
+                }
+                break;
+
             case Mod::PUSH:
             case Mod::PUSH_ALL:
                 uassert( 10141,
                          "Cannot apply $push/$pushAll modifier to non-array",
                          e.type() == Array || e.eoo() );
+
+                // Currently, we require the base array of a $sort to be made of
+                // objects (as opposed to base types).
+                if ( !e.eoo() && m.isEach() && m.isSliceAndSort() ) {
+                    BSONObjIterator i( e.embeddedObject() );
+                    while ( i.more() ) {
+                        BSONElement arrayItem = i.next();
+                        uassert( 16638,
+                                 "$sort can only be applied to an array of objects",
+                                 arrayItem.type() == Object );
+                    }
+                }
                 mss->amIInPlacePossible( false );
                 break;
 
@@ -550,7 +725,28 @@ namespace mongo {
         return mss;
     }
 
-    void ModState::appendForOpLog( BSONObjBuilder& b ) const {
+    const char* ModState::getOpLogName() const {
+        if ( dontApply ) {
+            return NULL;
+        }
+
+        if ( incType ) {
+            return "$set";
+        }
+
+        if ( m->op == Mod::RENAME_FROM ) {
+            return "$unset";
+        }
+
+        if ( m->op == Mod::RENAME_TO ) {
+            return "$set";
+        }
+
+        return fixedOpName ? fixedOpName : Mod::modNames[op()];
+    }
+
+
+    void ModState::appendForOpLog( BSONObjBuilder& bb ) const {
         // dontApply logic is deprecated for all but $rename.
         if ( dontApply ) {
             return;
@@ -559,23 +755,18 @@ namespace mongo {
         if ( incType ) {
             DEBUGUPDATE( "\t\t\t\t\t appendForOpLog inc fieldname: " << m->fieldName
                          << " short:" << m->shortFieldName );
-            BSONObjBuilder bb( b.subobjStart( "$set" ) );
             appendIncValue( bb , true );
-            bb.done();
             return;
         }
 
         if ( m->op == Mod::RENAME_FROM ) {
             DEBUGUPDATE( "\t\t\t\t\t appendForOpLog RENAME_FROM fieldName:" << m->fieldName );
-            BSONObjBuilder bb( b.subobjStart( "$unset" ) );
             bb.append( m->fieldName, 1 );
-            bb.done();
             return;
         }
 
         if ( m->op == Mod::RENAME_TO ) {
             DEBUGUPDATE( "\t\t\t\t\t appendForOpLog RENAME_TO fieldName:" << m->fieldName );
-            BSONObjBuilder bb( b.subobjStart( "$set" ) );
             bb.appendAs( newVal, m->fieldName );
             return;
         }
@@ -586,13 +777,10 @@ namespace mongo {
                      << " fn: " << m->fieldName );
 
         if (strcmp(name, "$unset") == 0) {
-            BSONObjBuilder bb(b.subobjStart(name));
             bb.append(m->fieldName, 1);
-            bb.done();
             return;
         }
 
-        BSONObjBuilder bb( b.subobjStart( name ) );
         if ( fixed ) {
             bb.appendAs( *fixed , m->fieldName );
         }
@@ -606,7 +794,32 @@ namespace mongo {
         else {
             bb.appendAs( m->elt , m->fieldName );
         }
-        bb.done();
+
+    }
+
+    typedef map<string, vector<ModState*> > NamedModMap;
+
+    BSONObj ModSetState::getOpLogRewrite() const {
+        NamedModMap names;
+        for ( ModStateHolder::const_iterator i = _mods.begin(); i != _mods.end(); ++i ) {
+            const char* name = i->second->getOpLogName();
+            if ( ! name )
+                continue;
+            names[name].push_back( i->second.get() );
+        }
+
+        BSONObjBuilder b;
+        for ( NamedModMap::const_iterator i = names.begin();
+              i != names.end();
+              ++i ) {
+            BSONObjBuilder bb( b.subobjStart( i->first ) );
+            const vector<ModState*>& mods = i->second;
+            for ( unsigned j = 0; j < mods.size(); j++ ) {
+                mods[j]->appendForOpLog( bb );
+            }
+            bb.doneFast();
+        }
+        return b.obj();
     }
 
     string ModState::toString() const {
@@ -684,6 +897,10 @@ namespace mongo {
                     BSONElementManipulator( m.old ).replaceTypeAndValue( m.m->elt );
                 break;
 
+            case Mod::SET_ON_INSERT:
+                // this should have been handled by prepare
+                break;
+
             default:
                 uassert( 13478 ,  "can't apply mod in place - shouldn't have gotten here" , 0 );
             }
@@ -705,12 +922,13 @@ namespace mongo {
             modState.fixedOpName = "$unset";
             return;
 
-        // $rename may involve dotted path creation, so we want to make sure we're not
-        // creating a path here for a rename that's a no-op. In other words if we're
-        // issuing a {$rename: {a.b : c.d} } that's a no-op, we don't want to create
-        // the a and c paths here. See test NestedNoName in the 'repl' suite.
+        // $rename/$setOnInsert may involve dotted path creation, so we want to make sure we're
+        // not creating a path here for a rename that's a no-op. In other words if we're
+        // issuing a {$rename: {a.b : c.d} } that's a no-op, we don't want to create the a and
+        // c paths here. See test NestedNoName in the 'repl' suite.
         case Mod::RENAME_FROM:
         case Mod::RENAME_TO:
+        case Mod::SET_ON_INSERT:
             if (modState.dontApply) {
                 return;
             }
@@ -915,6 +1133,40 @@ namespace mongo {
         return ss.str();
     }
 
+    bool ModSetState::isUpdateIndexedSlow() const {
+        // There may be indices over fields for which Mods are no-ops. In other words, if a
+        // Mod touches an index field but that Mod is a no-op, this update may be
+        // considered one that does not update indices.
+        if ( _numIndexMaybeUpdated == 0 ) {
+            return false;
+        }
+        else {
+            for ( ModStateHolder::const_iterator it = _mods.begin();
+                  it != _mods.end();
+                  ++it ) {
+                const Mod* m = it->second->m;
+                shared_ptr<ModState> ms = it->second;
+
+                switch ( m->op ) {
+                case Mod::SET_ON_INSERT:
+                case Mod::RENAME_FROM:
+                case Mod::RENAME_TO:
+                    if ( m->isIndexed && !ms->dontApply ) {
+                        return true;
+                    }
+                    break;
+
+                default:
+                    // no-op
+                    break;
+                }
+            }
+
+            return false;
+        }
+    }
+
+
     BSONObj ModSet::createNewFromQuery( const BSONObj& query ) {
         BSONObj newObj;
 
@@ -932,9 +1184,16 @@ namespace mongo {
                     // this can be a query piece
                     // or can be a dbref or something
 
-                    int op = e.embeddedObject().firstElement().getGtLtOp( -1 );
-                    if ( op >= 0 ) {
-                        // this means this is a $gt type filter, so don't make part of the new object
+                    int op = e.embeddedObject().firstElement().getGtLtOp();
+                    if ( op > 0 ) {
+                        // This means this is a $gt type filter, so don't make it part of the new
+                        // object.
+                        continue;
+                    }
+
+                    if ( str::equals( e.embeddedObject().firstElement().fieldName(), "$not" ) ) {
+                        // A $not filter operator is not detected in getGtLtOp() and should not
+                        // become part of the new object.
                         continue;
                     }
                 }
@@ -945,7 +1204,7 @@ namespace mongo {
             newObj = bb.obj();
         }
 
-        auto_ptr<ModSetState> mss = prepare( newObj );
+        auto_ptr<ModSetState> mss = prepare( newObj, true /* this is an insertion */ );
 
         if ( mss->canApplyInPlace() )
             mss->applyModsInPlace( false );
@@ -964,12 +1223,12 @@ namespace mongo {
        { $pullAll : { a:[99,1010] } }
        NOTE: MODIFIES source from object!
     */
-    ModSet::ModSet(
-        const BSONObj& from ,
-        const set<string>& idxKeys,
-        const set<string>* backgroundKeys,
-        bool forReplication)
-        : _isIndexed(0) , _hasDynamicArray( false ) {
+    ModSet::ModSet( const BSONObj& from ,
+                    const IndexPathSet& idxKeys,
+                    bool forReplication )
+        : _numIndexMaybeUpdated( 0 )
+        , _numIndexAlwaysUpdated( 0 )
+        , _hasDynamicArray( false ) {
 
         BSONObjIterator it(from);
 
@@ -1013,6 +1272,100 @@ namespace mongo {
                          "Modifier $pushAll/pullAll allowed for arrays only",
                          f.type() == Array || ( op != Mod::PUSH_ALL && op != Mod::PULL_ALL ) );
 
+                // Check whether $each, $slice, and $sort syntax for $push is correct.
+                if ( ( op == Mod::PUSH ) && ( f.type() == Object ) ) {
+                    BSONObj pushObj = f.embeddedObject();
+                    if ( pushObj.nFields() > 0 &&
+                         strcmp(pushObj.firstElement().fieldName(), "$each") == 0 ) {
+                        uassert( 16564,
+                                 "$each term needs to occur alone (or with $slice/$sort)",
+                                 pushObj.nFields() <= 3 );
+                        uassert( 16565,
+                                 "$each requires an array value",
+                                 pushObj.firstElement().type() == Array );
+
+                        // If both $slice and $sort are present, they may be switched.
+                        if ( pushObj.nFields() > 1 ) {
+                            BSONObjIterator i( pushObj );
+                            i.next();
+
+                            bool seenSlice = false;
+                            bool seenSort = false;
+                            while ( i.more() ) {
+                                BSONElement nextElem = i.next();
+
+                                if ( str::equals( nextElem.fieldName(), "$slice" ) ) {
+                                    uassert( 16567, "$slice appeared twice", !seenSlice);
+                                    seenSlice = true;
+                                    uassert( 16568,
+                                             "$slice value must be a numeric integer",
+                                             nextElem.type() == NumberInt ||
+                                             nextElem.type() == NumberLong ||
+                                             (nextElem.type() == NumberDouble &&
+                                              nextElem.numberDouble() ==
+                                              (long long)nextElem.numberDouble() ) );
+                                    uassert( 16640,
+                                             "$slice value must be negative or zero",
+                                             nextElem.number() <= 0 );
+                                }
+                                else if ( str::equals( nextElem.fieldName(), "$sort" ) ) {
+                                    uassert( 16647, "$sort appeared twice", !seenSort );
+                                    seenSort = true;
+                                    uassert( 16648,
+                                             "$sort component of $push must be an object",
+                                             nextElem.type() == Object );
+
+                                    BSONObjIterator j( nextElem.embeddedObject() );
+                                    while ( j.more() ) {
+                                        BSONElement fieldSortElem = j.next();
+                                        uassert( 16641,
+                                                 "$sort elements' values  must either 1 or -1",
+                                                 ( fieldSortElem.type() == NumberInt ||
+                                                   fieldSortElem.type() == NumberLong ||
+                                                   ( fieldSortElem.type() == NumberDouble &&
+                                                     fieldSortElem.numberDouble() ==
+                                                     (long long) fieldSortElem.numberDouble() ) ) &&
+                                                 ( fieldSortElem.Number() == 1 ||
+                                                   fieldSortElem.Number() == -1 ) );
+
+                                        FieldRef sortField;
+                                        sortField.parse( fieldSortElem.fieldName() );
+                                        uassert( 16690,
+                                                 "$sort field cannot be empty",
+                                                 sortField.numParts() > 0 );
+
+                                        for ( size_t i = 0; i < sortField.numParts(); i++ ) {
+                                            uassert( 16691,
+                                                     "empty field in dotted sort pattern",
+                                                     sortField.getPart( i ).size() > 0 );
+                                        }
+                                    }
+
+                                    // Finally, check if the $each is made of objects (as opposed
+                                    // to basic types). Currently, $sort only supports operating
+                                    // on arrays of objects.
+                                    BSONObj eachArray = pushObj.firstElement().embeddedObject();
+                                    BSONObjIterator k( eachArray );
+                                    while ( k.more() ) {
+                                        BSONElement eachItem = k.next();
+                                        uassert( 16642,
+                                                 "$sort requires $each to be an array of objects",
+                                                 eachItem.type() == Object );
+                                    }
+
+                                }
+                                else {
+                                    uasserted( 16643,
+                                               "$each term takes only $slice (and optionally "
+                                               "$sort) as complements" );
+                                }
+                            }
+
+                            uassert( 16644, "cannot have a $sort without a $slice", seenSlice );
+                        }
+                    }
+                }
+
                 if ( op == Mod::RENAME_TO ) {
                     uassert( 13494, "$rename target must be a string", f.type() == String );
                     const char* target = f.valuestr();
@@ -1059,13 +1412,13 @@ namespace mongo {
                     Mod from;
                     from.init( Mod::RENAME_FROM, f , forReplication );
                     from.setFieldName( fieldName );
-                    updateIsIndexed( from, idxKeys, backgroundKeys );
+                    setIndexedStatus( from, idxKeys );
                     _mods[ from.fieldName ] = from;
 
                     Mod to;
                     to.init( Mod::RENAME_TO, f , forReplication );
                     to.setFieldName( target );
-                    updateIsIndexed( to, idxKeys, backgroundKeys );
+                    setIndexedStatus( to, idxKeys );
                     _mods[ to.fieldName ] = to;
 
                     DEBUGUPDATE( "\t\t " << fieldName << "\t" << from.fieldName << "\t" << to.fieldName );
@@ -1077,7 +1430,7 @@ namespace mongo {
                 Mod m;
                 m.init( op , f , forReplication );
                 m.setFieldName( f.fieldName() );
-                updateIsIndexed( m, idxKeys, backgroundKeys );
+                setIndexedStatus( m, idxKeys );
                 _mods[m.fieldName] = m;
 
                 DEBUGUPDATE( "\t\t " << fieldName << "\t" << m.fieldName << "\t" << _hasDynamicArray );
@@ -1088,7 +1441,8 @@ namespace mongo {
 
     ModSet* ModSet::fixDynamicArray( const string& elemMatchKey ) const {
         ModSet* n = new ModSet();
-        n->_isIndexed = _isIndexed;
+        n->_numIndexMaybeUpdated = _numIndexMaybeUpdated;
+        n->_numIndexAlwaysUpdated = _numIndexAlwaysUpdated;
         n->_hasDynamicArray = _hasDynamicArray;
         for ( ModHolder::const_iterator i=_mods.begin(); i!=_mods.end(); i++ ) {
             string s = i->first;
@@ -1108,58 +1462,34 @@ namespace mongo {
         return n;
     }
 
-    void ModSet::updateIsIndexed( const set<string>& idxKeys, const set<string>* backgroundKeys ) {
-        for ( ModHolder::const_iterator i = _mods.begin(); i != _mods.end(); ++i )
-            updateIsIndexed( i->second, idxKeys , backgroundKeys );
+    void ModSet::setIndexedStatus( const IndexPathSet& idxKeys ) {
+        for ( ModHolder::iterator i = _mods.begin(); i != _mods.end(); ++i )
+            setIndexedStatus( i->second, idxKeys );
     }
 
-    bool getCanonicalIndexField( const string& fullName, string* out ) {
-        // check if fieldName contains ".$" or ".###" substrings (#=digit) and skip them
-        if ( fullName.find( '.' ) == string::npos )
-            return false;
+    void ModSet::setIndexedStatus( Mod& m, const IndexPathSet& idxKeys ) {
+        if ( idxKeys.mightBeIndexed( m.fieldName ) ) {
+            m.isIndexed = true;
 
-        bool modified = false;
+            // Some mods may be no-ops depending on the document they are applied
+            // on. Determining how many indices will actually be used can only be
+            // determined for sure after looking at that target document.
+            switch ( m.op ) {
 
-        StringBuilder buf;
-        for ( size_t i=0; i<fullName.size(); i++ ) {
+            case Mod::SET_ON_INSERT:
+            case Mod::RENAME_FROM:
+            case Mod::RENAME_TO:
+                _numIndexMaybeUpdated++;
+                break;
 
-            char c = fullName[i];
+            default:
+                _numIndexAlwaysUpdated++;
 
-            if ( c != '.' ) {
-                buf << c;
-                continue;
             }
-
-            // check for ".$", skip if present
-            if ( fullName[i+1] == '$' ) {
-                i++;
-                modified = true;
-                continue;
-            }
-
-            // check for ".###" for any number of digits (no letters)
-            if ( isdigit( fullName[i+1] ) ) {
-                size_t j = i;
-                // skip digits
-                while ( j+1 < fullName.size() && isdigit( fullName[j+1] ) )
-                    j++;
-
-                if ( j+1 == fullName.size() || fullName[j+1] == '.' ) {
-                    // only digits found, skip forward
-                    i = j;
-                    modified = true;
-                    continue;
-                }
-            }
-
-            buf << c;
         }
-
-        if ( !modified )
-            return false;
-
-        *out = buf.str();
-        return true;
+        else {
+            m.isIndexed = false;
+        }
     }
 
 
