@@ -18,25 +18,27 @@
 
 #include "pch.h"
 
-#include "../s/d_chunk_manager.h"
-
-#include "dbtests.h"
+#include "mongo/dbtests/dbtests.h"
+#include "mongo/s/chunk_version.h"
+#include "mongo/s/d_chunk_manager.h"
+#include "mongo/s/type_chunk.h"
+#include "mongo/s/type_collection.h"
 
 namespace {
 
     class BasicTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1)) <<
+                                      CollectionType::unique(false));
 
             // single-chunk collection
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKey" <<
-                                                 "ns"  << "test.foo" <<
-                                                 "min" << BSON( "a" << MINKEY ) <<
-                                                 "max" << BSON( "a" << MAXKEY ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(BSON("a" << MINKEY)) <<
+                                               ChunkType::max(BSON("a" << MAXKEY))));
 
             ShardChunkManager s ( collection , chunks );
 
@@ -52,16 +54,18 @@ namespace {
     class BasicCompoundTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
 
             // single-chunk collection
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKeyb_MinKey" <<
-                                                 "ns"  << "test.foo" <<
-                                                 "min" << BSON( "a" << MINKEY << "b" << MINKEY ) <<
-                                                 "max" << BSON( "a" << MAXKEY << "b" << MAXKEY ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKeyb_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(BSON("a" << MINKEY <<
+                                                                     "b" << MINKEY)) <<
+                                               ChunkType::max(BSON("a" << MAXKEY <<
+                                                                     "b" << MINKEY))));
 
             ShardChunkManager s ( collection , chunks );
 
@@ -79,25 +83,27 @@ namespace {
     class RangeTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "x.y" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("x.y") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1)) <<
+                                      CollectionType::unique(false));
 
             // 3-chunk collection, 2 of them being contiguous
             // [min->10) , [10->20) , <gap> , [30->max)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "x.y-a_MinKey" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << MINKEY ) <<
-                                                 "max" << BSON( "a" << 10 ) ) <<
-                                           BSON( "_id" << "x.y-a_10" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 10 ) <<
-                                                 "max" << BSON( "a" << 20 ) ) <<
-                                           BSON( "_id" << "x.y-a_30" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 30 ) <<
-                                                 "max" << BSON( "a" << MAXKEY ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("x.y-a_MinKey") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << MINKEY)) <<
+                                               ChunkType::max(BSON("a" << 10))) <<
+
+                                          BSON(ChunkType::name("x.y-a_10") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 10)) <<
+                                               ChunkType::max(BSON("a" << 20))) <<
+
+                                          BSON(ChunkType::name("x.y-a_30") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 30)) <<
+                                               ChunkType::max(BSON("a" << MAXKEY))));
 
             ShardChunkManager s ( collection , chunks );
 
@@ -117,11 +123,11 @@ namespace {
     class GetNextTests {
     public:
         void run() {
+            BSONObj collection = BSON(CollectionType::ns("x.y") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1)) <<
+                                      CollectionType::unique(false));
 
-            BSONObj collection = BSON( "_id"     << "x.y" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 ) <<
-                                       "unique"  << false );
             // empty collection
             BSONArray chunks1 = BSONArray();
             ShardChunkManager s1( collection , chunks1 );
@@ -141,10 +147,10 @@ namespace {
             // [10->20]
             BSONObj key_a10 = BSON( "a" << 10 );
             BSONObj key_a20 = BSON( "a" << 20 );
-            BSONArray chunks2 = BSON_ARRAY( BSON( "_id" << "x.y-a_10" <<
-                                                  "ns"  << "x.y" <<
-                                                  "min" << key_a10 <<
-                                                  "max" << key_a20 ) );
+            BSONArray chunks2 = BSON_ARRAY(BSON(ChunkType::name("x.y-a_10") <<
+                                                ChunkType::ns("x.y") <<
+                                                ChunkType::min(key_a10) <<
+                                                ChunkType::max(key_a20)));
             ShardChunkManager s2( collection , chunks2 );
             ASSERT( s2.getNextChunk( empty , &foundMin , &foundMax ) );
             ASSERT( foundMin.woCompare( key_a10 ) == 0 );
@@ -155,18 +161,18 @@ namespace {
             BSONObj key_a30 = BSON( "a" << 30 );
             BSONObj key_min = BSON( "a" << MINKEY );
             BSONObj key_max = BSON( "a" << MAXKEY );
-            BSONArray chunks3 = BSON_ARRAY( BSON( "_id" << "x.y-a_MinKey" <<
-                                                  "ns"  << "x.y" <<
-                                                  "min" << key_min <<
-                                                  "max" << key_a10  ) <<
-                                            BSON( "_id" << "x.y-a_10" <<
-                                                  "ns"  << "x.y" <<
-                                                  "min" << key_a10  <<
-                                                  "max" << key_a20  ) <<
-                                            BSON( "_id" << "x.y-a_30" <<
-                                                  "ns"  << "x.y" <<
-                                                  "min" << key_a30  <<
-                                                  "max" << key_max  ) );
+            BSONArray chunks3 = BSON_ARRAY(BSON(ChunkType::name("x.y-a_MinKey") <<
+                                                ChunkType::ns("x.y") <<
+                                                ChunkType::min(key_min) <<
+                                                ChunkType::max(key_a10)) <<
+                                           BSON(ChunkType::name("x.y-a_10") <<
+                                                ChunkType::ns("x.y") <<
+                                                ChunkType::min(key_a10)  <<
+                                                ChunkType::max(key_a20)) <<
+                                           BSON(ChunkType::name("x.y-a_30") <<
+                                                ChunkType::ns("x.y") <<
+                                                ChunkType::min(key_a30)  <<
+                                                ChunkType::max(key_max)));
             ShardChunkManager s3( collection , chunks3 );
             ASSERT( ! s3.getNextChunk( empty , &foundMin , &foundMax ) ); // not eof
             ASSERT( foundMin.woCompare( key_min ) == 0 );
@@ -181,8 +187,8 @@ namespace {
     class DeletedTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << "true" );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(true));
 
             BSONArray chunks = BSONArray();
 
@@ -193,23 +199,24 @@ namespace {
     class ClonePlusTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
+
             // 1-chunk collection
             // [10,0-20,0)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKey" <<
-                                                 "ns"  << "test.foo" <<
-                                                 "min" << BSON( "a" << 10 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 20 << "b" << 0 ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(BSON("a" << 10 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 20 << "b" << 0))));
 
             ShardChunkManager s ( collection , chunks );
 
             // new chunk [20,0-30,0)
             BSONObj min = BSON( "a" << 20 << "b" << 0 );
             BSONObj max = BSON( "a" << 30 << "b" << 0 );
-            ShardChunkManagerPtr cloned( s.clonePlus( min , max , ShardChunkVersion( 1, 0, OID() ) /* TODO test version */ ) );
+            ShardChunkManagerPtr cloned( s.clonePlus( min , max , ChunkVersion( 1, 0, OID() ) /* TODO test version */ ) );
 
             BSONObj k1 = BSON( "a" << 5 << "b" << 0 );
             ASSERT( ! cloned->belongsToMe( k1 ) );
@@ -225,51 +232,52 @@ namespace {
     class ClonePlusExceptionTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
+
             // 1-chunk collection
             // [10,0-20,0)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKey" <<
-                                                 "ns"  << "test.foo" <<
-                                                 "min" << BSON( "a" << 10 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 20 << "b" << 0 ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(BSON("a" << 10 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 20 << "b" << 0))));
 
             ShardChunkManager s ( collection , chunks );
 
             // [15,0-25,0) overlaps [10,0-20,0)
             BSONObj min = BSON( "a" << 15 << "b" << 0 );
             BSONObj max = BSON( "a" << 25 << "b" << 0 );
-            ASSERT_THROWS( s.clonePlus ( min , max , ShardChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
+            ASSERT_THROWS( s.clonePlus ( min , max , ChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
         }
     };
 
     class CloneMinusTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "x.y" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("x.y") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
 
             // 2-chunk collection
             // [10,0->20,0) , <gap> , [30,0->40,0)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "x.y-a_10b_0" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 10 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 20 << "b" << 0 ) ) <<
-                                           BSON( "_id" << "x.y-a_30b_0" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 30 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 40 << "b" << 0 ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("x.y-a_10b_0") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 10 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 20 << "b" << 0))) <<
+                                          BSON(ChunkType::name("x.y-a_30b_0") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 30 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 40 << "b" << 0))));
 
             ShardChunkManager s ( collection , chunks );
 
             // deleting chunk [10,0-20,0)
             BSONObj min = BSON( "a" << 10 << "b" << 0 );
             BSONObj max = BSON( "a" << 20 << "b" << 0 );
-            ShardChunkManagerPtr cloned( s.cloneMinus( min , max , ShardChunkVersion( 1, 0, OID() ) /* TODO test version */ ) );
+            ShardChunkManagerPtr cloned( s.cloneMinus( min , max , ChunkVersion( 1, 0, OID() ) /* TODO test version */ ) );
 
             BSONObj k1 = BSON( "a" << 5 << "b" << 0 );
             ASSERT( ! cloned->belongsToMe( k1 ) );
@@ -287,52 +295,53 @@ namespace {
     class CloneMinusExceptionTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "x.y" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("x.y") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
 
             // 2-chunk collection
             // [10,0->20,0) , <gap> , [30,0->40,0)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "x.y-a_10b_0" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 10 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 20 << "b" << 0 ) ) <<
-                                           BSON( "_id" << "x.y-a_30b_0" <<
-                                                 "ns"  << "x.y" <<
-                                                 "min" << BSON( "a" << 30 << "b" << 0 ) <<
-                                                 "max" << BSON( "a" << 40 << "b" << 0 ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("x.y-a_10b_0") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 10 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 20 << "b" << 0))) <<
+                                          BSON(ChunkType::name("x.y-a_30b_0") <<
+                                               ChunkType::ns("x.y") <<
+                                               ChunkType::min(BSON("a" << 30 << "b" << 0)) <<
+                                               ChunkType::max(BSON("a" << 40 << "b" << 0))));
 
             ShardChunkManager s ( collection , chunks );
 
             // deleting non-existing chunk [25,0-28,0)
             BSONObj min1 = BSON( "a" << 25 << "b" << 0 );
             BSONObj max1 = BSON( "a" << 28 << "b" << 0 );
-            ASSERT_THROWS( s.cloneMinus( min1 , max1 , ShardChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
+            ASSERT_THROWS( s.cloneMinus( min1 , max1 , ChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
 
 
             // deletin an overlapping range (not exactly a chunk) [15,0-25,0)
             BSONObj min2 = BSON( "a" << 15 << "b" << 0 );
             BSONObj max2 = BSON( "a" << 25 << "b" << 0 );
-            ASSERT_THROWS( s.cloneMinus( min2 , max2 , ShardChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
+            ASSERT_THROWS( s.cloneMinus( min2 , max2 , ChunkVersion( 1, 0, OID() ) /* TODO test version */ ) , UserException );
         }
     };
 
     class CloneSplitTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
+
             // 1-chunk collection
             // [10,0-20,0)
             BSONObj min = BSON( "a" << 10 << "b" << 0 );
             BSONObj max = BSON( "a" << 20 << "b" << 0 );
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKey"
-                                                 << "ns"  << "test.foo"
-                                                 << "min" << min
-                                                 << "max" << max ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(min) <<
+                                               ChunkType::max(max)));
 
             ShardChunkManager s ( collection , chunks );
 
@@ -341,7 +350,7 @@ namespace {
             vector<BSONObj> splitKeys;
             splitKeys.push_back( split1 );
             splitKeys.push_back( split2 );
-            ShardChunkVersion version( 1 , 99, OID() ); // first chunk 1|99 , second 1|100
+            ChunkVersion version( 1 , 99, OID() ); // first chunk 1|99 , second 1|100
             ShardChunkManagerPtr cloned( s.cloneSplit( min , max , splitKeys , version ) );
 
             version.incMinor(); /* second chunk 1|100, first split point */
@@ -359,41 +368,42 @@ namespace {
     class CloneSplitExceptionTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 << "b" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1 << "b" << 1)) <<
+                                      CollectionType::unique(false));
+
             // 1-chunk collection
             // [10,0-20,0)
             BSONObj min = BSON( "a" << 10 << "b" << 0 );
             BSONObj max = BSON( "a" << 20 << "b" << 0 );
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_MinKey"
-                                                 << "ns"  << "test.foo"
-                                                 << "min" << min
-                                                 << "max" << max ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_MinKey") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(min) <<
+                                               ChunkType::max(max)));
 
             ShardChunkManager s ( collection , chunks );
 
             BSONObj badSplit = BSON( "a" << 5 << "b" << 0 );
             vector<BSONObj> splitKeys;
             splitKeys.push_back( badSplit );
-            ASSERT_THROWS( s.cloneSplit( min , max , splitKeys , ShardChunkVersion( 1, 0, OID() ) ) , UserException );
+            ASSERT_THROWS( s.cloneSplit( min , max , splitKeys , ChunkVersion( 1, 0, OID() ) ) , UserException );
 
             BSONObj badMax = BSON( "a" << 25 << "b" << 0 );
             BSONObj split = BSON( "a" << 15 << "b" << 0 );
             splitKeys.clear();
             splitKeys.push_back( split );
-            ASSERT_THROWS( s.cloneSplit( min , badMax, splitKeys , ShardChunkVersion( 1, 0, OID() ) ) , UserException );
+            ASSERT_THROWS( s.cloneSplit( min , badMax, splitKeys , ChunkVersion( 1, 0, OID() ) ) , UserException );
         }
     };
 
     class EmptyShardTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 ) <<
-                                       "unique"  << false );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1)) <<
+                                      CollectionType::unique(false));
 
             // no chunks on this shard
             BSONArray chunks;
@@ -401,7 +411,7 @@ namespace {
             // shard can have zero chunks for an existing collection
             // version should be 0, though
             ShardChunkManager s( collection , chunks );
-            ASSERT_EQUALS( s.getVersion().toLong() , ShardChunkVersion( 0, 0, OID() ).toLong() );
+            ASSERT_EQUALS( s.getVersion().toLong() , ChunkVersion( 0, 0, OID() ).toLong() );
             ASSERT_EQUALS( s.getNumChunks() , 0u );
         }
     };
@@ -409,34 +419,34 @@ namespace {
     class LastChunkTests {
     public:
         void run() {
-            BSONObj collection = BSON( "_id"     << "test.foo" <<
-                                       "dropped" << false <<
-                                       "key"     << BSON( "a" << 1 ) <<
-                                       "unique"  << false  );
+            BSONObj collection = BSON(CollectionType::ns("test.foo") <<
+                                      CollectionType::dropped(false) <<
+                                      CollectionType::keyPattern(BSON("a" << 1)) <<
+                                      CollectionType::unique(false));
 
             // 1-chunk collection
             // [10->20)
-            BSONArray chunks = BSON_ARRAY( BSON( "_id" << "test.foo-a_10" <<
-                                                 "ns"  << "test.foo" <<
-                                                 "min" << BSON( "a" << 10 ) <<
-                                                 "max" << BSON( "a" << 20 ) ) );
+            BSONArray chunks = BSON_ARRAY(BSON(ChunkType::name("test.foo-a_10") <<
+                                               ChunkType::ns("test.foo") <<
+                                               ChunkType::min(BSON("a" << 10)) <<
+                                               ChunkType::max(BSON("a" << 20))));
 
             ShardChunkManager s( collection , chunks );
             BSONObj min = BSON( "a" << 10 );
             BSONObj max = BSON( "a" << 20 );
 
             // if we remove the only chunk, the only version accepted is 0
-            ShardChunkVersion nonZero = ShardChunkVersion( 99, 0, OID() );
+            ChunkVersion nonZero = ChunkVersion( 99, 0, OID() );
             ASSERT_THROWS( s.cloneMinus( min , max , nonZero ) , UserException );
-            ShardChunkManagerPtr empty( s.cloneMinus( min , max , ShardChunkVersion( 0, 0, OID() ) ) );
-            ASSERT_EQUALS( empty->getVersion().toLong() , ShardChunkVersion( 0, 0, OID() ).toLong() );
+            ShardChunkManagerPtr empty( s.cloneMinus( min , max , ChunkVersion( 0, 0, OID() ) ) );
+            ASSERT_EQUALS( empty->getVersion().toLong() , ChunkVersion( 0, 0, OID() ).toLong() );
             ASSERT_EQUALS( empty->getNumChunks() , 0u );
             BSONObj k = BSON( "a" << 15 << "b" << 0 );
             ASSERT( ! empty->belongsToMe( k ) );
 
             // we can add a chunk to an empty manager
             // version should be provided
-            ASSERT_THROWS( empty->clonePlus( min , max , ShardChunkVersion( 0, 0, OID() ) ) , UserException );
+            ASSERT_THROWS( empty->clonePlus( min , max , ChunkVersion( 0, 0, OID() ) ) , UserException );
             ShardChunkManagerPtr cloned( empty->clonePlus( min , max , nonZero ) );
             ASSERT_EQUALS( cloned->getVersion().toLong(), nonZero.toLong() );
             ASSERT_EQUALS( cloned->getNumChunks() , 1u );
