@@ -19,23 +19,34 @@
  * ReplicaSetMonitor::selectNode and TagSet
  */
 
-#include <vector>
-
+#include "mongo/client/connpool.h"
+#include "mongo/client/dbclientinterface.h"
 #include "mongo/client/dbclient_rs.h"
+#include "mongo/dbtests/mock/mock_conn_registry.h"
+#include "mongo/dbtests/mock/mock_replica_set.h"
 #include "mongo/unittest/unittest.h"
 
-namespace {
-    using std::vector;
-    using boost::scoped_ptr;
+#include <vector>
 
-    using mongo::BSONObj;
-    using mongo::BSONArray;
-    using mongo::BSONArrayBuilder;
-    using mongo::ReplicaSetMonitor;
-    using mongo::HostAndPort;
-    using mongo::ReadPreference;
-    using mongo::TagSet;
+using std::vector;
+using std::string;
+using boost::scoped_ptr;
 
+using mongo::BSONObj;
+using mongo::BSONObjBuilder;
+using mongo::BSONArray;
+using mongo::BSONArrayBuilder;
+using mongo::BSONElement;
+using mongo::ConnectionString;
+using mongo::HostAndPort;
+using mongo::MockReplicaSet;
+using mongo::ReadPreference;
+using mongo::ReplicaSetMonitor;
+using mongo::ReplicaSetMonitorPtr;
+using mongo::ScopedDbConnection;
+using mongo::TagSet;
+
+namespace mongo_test {
     const BSONObj SampleIsMasterDoc = BSON("tags"
                                             << BSON("dc" << "NYC"
                                                     << "p" << "2"
@@ -1403,6 +1414,33 @@ namespace {
         ASSERT_EQUALS("b", lastHost.host());
     }
 
+    TEST(TagSet, CopyConstructor) {
+        TagSet* copy;
+
+        {
+            BSONArrayBuilder builder;
+            builder.append(BSON("dc" << "nyc"));
+            builder.append(BSON("priority" << "1"));
+            TagSet original(builder.arr());
+
+            original.next();
+
+            copy = new TagSet(original);
+        }
+
+        ASSERT_FALSE(copy->isExhausted());
+        ASSERT(copy->getCurrentTag().equal(BSON("dc" << "nyc")));
+        copy->next();
+
+        ASSERT_FALSE(copy->isExhausted());
+        ASSERT(copy->getCurrentTag().equal(BSON("priority" << "1")));
+        copy->next();
+
+        ASSERT(copy->isExhausted());
+
+        delete copy;
+    }
+
     TEST(TagSet, NearestMultiTagsNoMatch) {
         vector<ReplicaSetMonitor::Node> nodes =
                 NodeSetFixtures::getThreeMemberWithTags();
@@ -1475,4 +1513,106 @@ namespace {
         ASSERT_THROWS(tags.getCurrentTag(), mongo::AssertionException);
 #endif
     }
+
+    // TODO: Port these existing tests here: replmonitor_bad_seed.js, repl_monitor_refresh.js
+
+    /**
+     * Warning: Tests running this fixture cannot be run in parallel with other tests
+     * that uses ConnectionString::setConnectionHook
+     */
+    class ReplicaSetMonitorTest: public mongo::unittest::Test {
+    protected:
+        void setUp() {
+            _replSet.reset(new MockReplicaSet("test", 3));
+            _originalConnectionHook = ConnectionString::getConnectionHook();
+            ConnectionString::setConnectionHook(
+                    mongo::MockConnRegistry::get()->getConnStrHook());
+        }
+
+        void tearDown() {
+            ConnectionString::setConnectionHook(_originalConnectionHook);
+            ReplicaSetMonitor::remove(_replSet->getSetName(), true);
+            _replSet.reset();
+            mongo::ScopedDbConnection::clearPool();
+        }
+
+        MockReplicaSet* getReplSet() {
+            return _replSet.get();
+        }
+
+    private:
+        ConnectionString::ConnectionHook* _originalConnectionHook;
+        boost::scoped_ptr<MockReplicaSet> _replSet;
+    };
+
+    TEST_F(ReplicaSetMonitorTest, SeedWithPriOnlySecDown) {
+        // Test to make sure that the monitor doesn't crash when
+        // ConnectionString::connect returns NULL
+        MockReplicaSet* replSet = getReplSet();
+        replSet->kill(replSet->getSecondaries());
+
+        // Create a monitor with primary as the only seed list and the two secondaries
+        // down so a NULL connection object will be stored for these secondaries in
+        // the _nodes vector.
+        const string replSetName(replSet->getSetName());
+        vector<HostAndPort> seedList;
+        seedList.push_back(HostAndPort(replSet->getPrimary()));
+        ReplicaSetMonitor::createIfNeeded(replSetName, seedList);
+
+        replSet->kill(replSet->getPrimary());
+
+        ReplicaSetMonitorPtr monitor = ReplicaSetMonitor::get(replSet->getSetName());
+        // Trigger calls to Node::getConnWithRefresh
+        monitor->check(true);
+    }
+
+    // Stress test case for a node that is previously a primary being removed from the set.
+    // This test goes through configurations with different positions for the primary node
+    // in the host list returned from the isMaster command. The test here is to make sure
+    // that the ReplicaSetMonitor will not crash under these situations.
+    TEST(ReplicaSetMonitorTest, PrimaryRemovedFromSetStress) {
+        const size_t NODE_COUNT = 5;
+        MockReplicaSet replSet("test", NODE_COUNT);
+        ConnectionString::ConnectionHook* originalConnHook =
+                ConnectionString::getConnectionHook();
+        ConnectionString::setConnectionHook(mongo::MockConnRegistry::get()->getConnStrHook());
+
+        const string replSetName(replSet.getSetName());
+        vector<HostAndPort> seedList;
+        seedList.push_back(HostAndPort(replSet.getPrimary()));
+        ReplicaSetMonitor::createIfNeeded(replSetName, seedList);
+
+        const MockReplicaSet::ReplConfigMap origConfig = replSet.getReplConfig();
+        mongo::ReplicaSetMonitorPtr replMonitor = ReplicaSetMonitor::get(replSetName);
+
+        for (size_t idxToRemove = 0; idxToRemove < NODE_COUNT; idxToRemove++) {
+            MockReplicaSet::ReplConfigMap newConfig = origConfig;
+
+            replSet.setConfig(origConfig);
+            replMonitor->check(true); // Make sure the monitor sees the change
+
+            string hostToRemove;
+            {
+                BSONObjBuilder monitorStateBuilder;
+                replMonitor->appendInfo(monitorStateBuilder);
+                BSONObj monitorState = monitorStateBuilder.done();
+
+                BSONElement hostsElem = monitorState["hosts"];
+                BSONElement addrElem = hostsElem[mongo::str::stream() << idxToRemove]["addr"];
+                hostToRemove = addrElem.String();
+            }
+
+            replSet.setPrimary(hostToRemove);
+            replMonitor->check(true); // Make sure the monitor sees the new primary
+
+            newConfig.erase(hostToRemove);
+            replSet.setConfig(newConfig);
+            replSet.setPrimary(newConfig.begin()->first);
+            replMonitor->check(true); // Force refresh -> should not crash
+        }
+
+        ReplicaSetMonitor::remove(replSet.getSetName(), true);
+        ConnectionString::setConnectionHook(originalConnHook);
+        mongo::ScopedDbConnection::clearPool();
+    }
 }
