@@ -1771,6 +1771,8 @@ namespace mongo {
                              int options, string &errmsg,
                              BSONObjBuilder &result, bool fromRepl);
 
+            virtual bool passOptions() const { return true; }
+
         private:
             
         };
@@ -1815,7 +1817,7 @@ namespace mongo {
             */
             DBConfigPtr conf(grid.getDBConfig(dbName , false));
             if (!conf || !conf->isShardingEnabled() || !conf->isSharded(fullns))
-                return passthrough(conf, cmdObj, result);
+                return passthrough(conf, cmdObj, options, result);
 
             /* split the pipeline into pieces for mongods and this mongos */
             intrusive_ptr<Pipeline> pShardPipeline(
