@@ -27,6 +27,12 @@
  *        Note: For both formats, a special boolean property 'arbiter' can be
  *          specified to denote a member is an arbiter.
  * 
+ *     nodeOptions {Object}: Options to apply to all nodes in the replica set.
+ *        Format for Object:
+ *          { cmdline-param-with-no-arg : "",
+ *            param-with-arg : arg }
+ *        This turns into "mongod --cmdline-param-with-no-arg --param-with-arg arg" 
+ *  
  *     oplogSize {number}: Default: 40
  *     useSeedList {boolean}: Use the connection string format of this set
  *        as the replica set name (overrides the name property). Default: false
@@ -59,17 +65,22 @@ ReplSetTest = function( opts ){
     if( isObject( this.numNodes ) ){
         var len = 0
         for( var i in this.numNodes ){
-            var options = this.nodeOptions[ "n" + len ] = this.numNodes[i]
-            if( i.startsWith( "a" ) ) options.arbiter = true
+            var options = this.nodeOptions[ "n" + len ] = Object.merge(opts.nodeOptions, 
+                                                                       this.numNodes[i]);
+            if( i.startsWith( "a" ) ) options.arbiter = true;
             len++
         }
         this.numNodes = len
     }
     else if( Array.isArray( this.numNodes ) ){
         for( var i = 0; i < this.numNodes.length; i++ )
-            this.nodeOptions[ "n" + i ] = this.numNodes[i]
+            this.nodeOptions[ "n" + i ] = Object.merge(opts.nodeOptions, this.numNodes[i]);
         this.numNodes = this.numNodes.length
     }
+    else {
+        for ( var i =0; i < this.numNodes; i++ )
+            this.nodeOptions[ "n" + i ] = opts.nodeOptions;
+    }
     
     if(this.bridged) {
         this.bridgePorts = [];
@@ -308,8 +319,8 @@ ReplSetTest.prototype.callIsMaster = function() {
       }
 
     }
-    catch(err) {
-      print("ReplSetTest Could not call ismaster on node " + i);
+    catch (err) {
+      print("ReplSetTest Could not call ismaster on node " + i + ": " + tojson(err));
     }
   }
 
@@ -369,28 +380,38 @@ ReplSetTest.awaitRSClientHosts = function( conn, host, hostOk, rs ) {
 }
 
 ReplSetTest.prototype.awaitSecondaryNodes = function( timeout ) {
-  var master = this.getMaster();
-  var slaves = this.liveNodes.slaves;
-  var len = slaves.length;
-
-  jsTest.attempt({context: this, timeout: 60000, desc: "Awaiting secondaries"}, function() {
-     var ready = true;
-     for(var i=0; i<len; i++) {
-       var isMaster = slaves[i].getDB("admin").runCommand({ismaster: 1});
-       var arbiter = isMaster['arbiterOnly'] == undefined ? false : isMaster['arbiterOnly'];
-       ready = ready && ( isMaster['secondary'] || arbiter );
-     }
-     return ready;
-  });
-}
+  this.getMaster(timeout); // Wait for a primary to be selected.
+  var tmo = timeout || 60000;
+  var replTest = this;
+  assert.soon(
+      function() {
+          replTest.getMaster(); // Reload who the current slaves are.
+          var slaves = replTest.liveNodes.slaves;
+          var len = slaves.length;
+          var ready = true;
+          for(var i=0; i<len; i++) {
+              var isMaster = slaves[i].getDB("admin").runCommand({ismaster: 1});
+              var arbiter = isMaster['arbiterOnly'] == undefined ? false : isMaster['arbiterOnly'];
+              ready = ready && ( isMaster['secondary'] || arbiter );
+          }
+          return ready;
+      }, "Awaiting secondaries", tmo);
+};
 
 ReplSetTest.prototype.getMaster = function( timeout ) {
   var tries = 0;
   var sleepTime = 500;
-  var t = timeout || 000;
+  var tmo = timeout || 60000;
   var master = null;
 
-  master = jsTest.attempt({context: this, timeout: 60000, desc: "Finding master"}, this.callIsMaster);
+  try {
+    master = jsTest.attempt({context: this, timeout: tmo, desc: "Finding master"}, this.callIsMaster);
+  }
+  catch (err) {
+    print("ReplSetTest getMaster failed: " + tojson(err));
+    printStackTrace();
+    throw err;
+  }
   return master;
 }
 
@@ -448,23 +469,23 @@ ReplSetTest.prototype.initiate = function( cfg , initCmd , timeout ) {
     var config  = cfg || this.getReplSetConfig();
     var cmd     = {};
     var cmdKey  = initCmd || 'replSetInitiate';
-    var timeout = timeout || 30000;
+    var tmo     = timeout || 30000;
     cmd[cmdKey] = config;
     printjson(cmd);
 
-    jsTest.attempt({context:this, timeout: timeout, desc: "Initiate replica set"}, function() {
+    jsTest.attempt({context:this, timeout: tmo, desc: "Initiate replica set"}, function() {
         var result = master.runCommand(cmd);
         printjson(result);
         return result['ok'] == 1;
     });
 
+    this.awaitSecondaryNodes(timeout);
+
     // Setup authentication if running test with authentication
     if (jsTestOptions().keyFile && !this.keyFile) {
-        if (!this.shardSvr) {
-            master = this.getMaster();
-            jsTest.addAuth(master);
-            jsTest.authenticateNodes(this.nodes);
-        }
+        master = this.getMaster();
+        jsTest.addAuth(master);
+        jsTest.authenticateNodes(this.nodes);
     }
 }
 
@@ -496,54 +517,78 @@ ReplSetTest.prototype.awaitReplication = function(timeout) {
 
     this.getLastOpTimeWritten();
 
-    print("ReplSetTest " + this.latest);
+    var name = this.liveNodes.master.toString().substr(14); // strip "connection to "
+    print("ReplSetTest awaitReplication: starting: timestamp for primary, " +
+          name + ", is " + tojson(this.latest));
 
     jsTest.attempt({context: this, timeout: timeout, desc: "awaiting replication"},
                  function() {
                      try {
-                         var synced = true;
-                         for(var i=0; i<this.liveNodes.slaves.length; i++) {
+                         print("ReplSetTest awaitReplication: checking secondaries against timestamp " +
+                               tojson(this.latest));
+                         var secondaryCount = 0;
+                         for (var i=0; i<this.liveNodes.slaves.length; i++) {
                              var slave = this.liveNodes.slaves[i];
 
                              // Continue if we're connected to an arbiter
-                             if(res = slave.getDB("admin").runCommand({replSetGetStatus: 1})) {
-                                 if(res.myState == 7) {
+                             if (res = slave.getDB("admin").runCommand({replSetGetStatus: 1})) {
+                                 if (res.myState == 7) {
                                      continue;
                                  }
                              }
 
+                             ++secondaryCount;
+                             var name = slave.toString().substr(14); // strip "connection to "
+                             print("ReplSetTest awaitReplication: checking secondary #" +
+                                   secondaryCount + ": " + name);
                              slave.getDB("admin").getMongo().setSlaveOk();
                              var log = slave.getDB("local")['oplog.rs'];
-                             if(log.find({}).sort({'$natural': -1}).limit(1).hasNext()) {
+                             if (log.find({}).sort({'$natural': -1}).limit(1).hasNext()) {
                                  var entry = log.find({}).sort({'$natural': -1}).limit(1).next();
-                                 printjson( entry );
                                  var ts = entry['ts'];
-                                 print("ReplSetTest await TS for " + slave + " is " + ts.t+":"+ts.i + " and latest is " + this.latest.t+":"+this.latest.i);
-
-                                 if (this.latest.t < ts.t || (this.latest.t == ts.t && this.latest.i < ts.i)) {
-                                     this.latest = this.liveNodes.master.getDB("local")['oplog.rs'].find({}).sort({'$natural': -1}).limit(1).next()['ts'];
+                                 if (this.latest.t < ts.t ||
+                                        (this.latest.t == ts.t && this.latest.i < ts.i)) {
+                                     this.latest = this.liveNodes.master.getDB("local")['oplog.rs'].
+                                                        find({}).
+                                                        sort({'$natural': -1}).
+                                                        limit(1).
+                                                        next()['ts'];
+                                     print("ReplSetTest awaitReplication: timestamp for " + name +
+                                           " is newer, resetting latest to " + tojson(this.latest));
+                                     return false;
                                  }
-
-                                 print("ReplSetTest await oplog size for " + slave + " is " + log.count());
-                                 synced = (synced && friendlyEqual(this.latest,ts))
+                                 if (!friendlyEqual(this.latest, ts)) {
+                                     print("ReplSetTest awaitReplication: timestamp for secondary #" +
+                                           secondaryCount + ", " + name + ", is " + tojson(ts) +
+                                           " but latest is " + tojson(this.latest));
+                                     print("ReplSetTest awaitReplication: last oplog entry (of " +
+                                           log.count() + ") for secondary #" + secondaryCount +
+                                           ", " + name + ", is " + tojsononeline(entry));
+                                     print("ReplSetTest awaitReplication: secondary #" +
+                                           secondaryCount + ", " + name + ", is NOT synced");
+                                     return false;
+                                 }
+                                 print("ReplSetTest awaitReplication: secondary #" +
+                                       secondaryCount + ", " + name + ", is synced");
                              }
                              else {
-                                 print( "ReplSetTest waiting for " + slave + " to have an oplog built." )
-                                 synced = false;
+                                 print("ReplSetTest awaitReplication: waiting for secondary #" +
+                                       secondaryCount + ", " + name + ", to have an oplog built");
+                                 return false;
                              }
                          }
 
-                         if(synced) {
-                             print("ReplSetTest await synced=" + synced);
-                         }
-                         return synced;
+                         print("ReplSetTest awaitReplication: finished: all " + secondaryCount +
+                               " secondaries synced at timestamp " + tojson(this.latest));
+                         return true;
                      }
                      catch (e) {
-                         print("ReplSetTest.awaitReplication: caught exception "+e);
+                         print("ReplSetTest awaitReplication: caught exception: " + e);
 
                          // we might have a new master now
                          this.getLastOpTimeWritten();
-
+                         print("ReplSetTest awaitReplication: resetting: timestamp for primary " +
+                               this.liveNodes.master + " is " + tojson(this.latest));
                          return false;
                      }
                  });
@@ -602,7 +647,7 @@ ReplSetTest.prototype.start = function( n , options , restart , wait ){
                  dbpath : "$set-$node" }
     
     defaults = Object.merge( defaults, ReplSetTest.nodeOptions || {} )
-        
+
     // TODO : should we do something special if we don't currently know about this node?
     n = this.getNodeId( n )
     
