@@ -18,13 +18,11 @@
 
 #include "pch.h"
 #include "db/json.h"
-
 #include "tool.h"
 #include "../util/text.h"
-
+#include "mongo/base/initializer.h"
 #include <fstream>
 #include <iostream>
-
 #include <boost/program_options.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/filesystem/operations.hpp>
@@ -273,7 +271,7 @@ public:
         ("type",po::value<string>() , "type of file to import.  default: json (json,csv,tsv)")
         ("file",po::value<string>() , "file to import from; if not specified stdin is used" )
         ("drop", "drop collection first " )
-        ("headerline","CSV,TSV only - use first line as headers")
+        ("headerline","first line in input file is a header (CSV and TSV only)")
         ("upsert", "insert or update objects that already exist" )
         ("upsertFields", po::value<string>(), "comma-separated fields for the query part of the upsert. You should make sure this is indexed" )
         ("stopOnError", "stop importing at first error rather than continuing" )
@@ -290,9 +288,32 @@ public:
         _doimport = true;
         _jsonArray = false;
     }
-
+    ;
     virtual void printExtraHelp( ostream & out ) {
         out << "Import CSV, TSV or JSON data into MongoDB.\n" << endl;
+        out << "When importing JSON documents, each document must be a separate line of the input file.\n";
+        out << "\nExample:\n";
+        out << "  mongoimport --host myhost --db my_cms --collection docs < mydocfile.json\n" << endl;
+    }
+
+    unsigned long long lastErrorFailures;
+
+    /** @return true if ok */
+    bool checkLastError() { 
+        string s = conn().getLastError();
+        if( !s.empty() ) { 
+            if( str::contains(s,"uplicate") ) {
+                // we don't want to return an error from the mongoimport process for
+                // dup key errors
+                log() << s << endl;
+            }
+            else {
+                lastErrorFailures++;
+                log() << "error: " << s << endl;
+                return false;
+            }
+        }
+        return true;
     }
 
     int run() {
@@ -330,8 +351,6 @@ public:
 
         LOG(1) << "ns: " << ns << endl;
 
-        auth();
-
         if ( hasParam( "drop" ) ) {
             log() << "dropping: " << ns << endl;
             conn().dropCollection( ns.c_str() );
@@ -393,7 +412,9 @@ public:
         LOG(1) << "filesize: " << fileSize << endl;
         ProgressMeter pm( fileSize );
         int num = 0;
+        int lastNumChecked = num;
         int errors = 0;
+        lastErrorFailures = 0;
         int len = 0;
         // buffer and line are only used when parsing a jsonArray
         boost::scoped_array<char> buffer(new char[BUF_SIZE+2]);
@@ -445,6 +466,13 @@ public:
                     else {
                         conn().insert( ns.c_str() , o );
                     }
+
+                    if( num < 10 ) { 
+                        // we absolutely want to check the first and last op of the batch. we do 
+                        // a few more as that won't be too time expensive.
+                        checkLastError();
+                        lastNumChecked = num;
+                    }
                 }
 
                 num++;
@@ -463,20 +491,31 @@ public:
             }
         }
 
-        log() << "imported " << ( num - headerRows ) << " objects" << endl;
+        // this is for two reasons: to wait for all operations to reach the server and be processed, and this will wait until all data reaches the server,
+        // and secondly to check if there were an error (on the last op)
+        if( lastNumChecked+1 != num ) { // avoid redundant log message if already reported above
+            log() << "check " << lastNumChecked << " " << num << endl;
+            checkLastError();
+        }
 
-        conn().getLastError();
+        bool hadErrors = lastErrorFailures || errors;
 
-        if ( errors == 0 )
+        // the message is vague on lastErrorFailures as we don't call it on every single operation. 
+        // so if we have a lastErrorFailure there might be more than just what has been counted.
+        log() << (lastErrorFailures ? "tried to import " : "imported ") << ( num - headerRows ) << " objects" << endl;
+
+        if ( !hadErrors )
             return 0;
 
-        error() << "encountered " << errors << " error" << ( errors == 1 ? "" : "s" ) << endl;
+        error() << "encountered " << (lastErrorFailures?"at least ":"") << lastErrorFailures+errors <<  " error(s)" << ( lastErrorFailures+errors == 1 ? "" : "s" ) << endl;
         return -1;
     }
 };
 
-int main( int argc , char ** argv ) {
+int main( int argc , char ** argv, char** envp ) {
+    mongo::runGlobalInitializersOrDie(argc, argv, envp);
     Import import;
     return import.main( argc , argv );
 }
+
 const int Import::BUF_SIZE(1024 * 1024 * 16);
