@@ -23,6 +23,11 @@
 
 namespace mongo {
 
+    /**
+     * Maximum accepted message size on the wire protocol.
+     */
+    const int MaxMessageSizeBytes = 48 * 1000 * 1000;
+
     class Message;
     class MessagingPort;
     class PiggyBackData;
@@ -91,7 +96,7 @@ namespace mongo {
         int messageLength; // total message size, including this
         int requestID;     // identifier for this message
         int responseTo;    // requestID from the original request
-        //   (used in reponses from db)
+        //   (used in responses from db)
         int opCode;
     };
 #pragma pack()
